#include "dialogdebugmonitor.h"
#include "ui_dialogdebugmonitor.h"

//#include "ip_cores.h"
#include "ip_cores_hdmi.h"

DialogDebugMonitor::DialogDebugMonitor(int debug_data_size, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DialogDebugMonitor)
{
    ui->setupUi(this);
    debug_data_size_ = debug_data_size;
    debug_log2file_request_ = false;

    local_oscillator_   = NULL;
    cic_filter_         = NULL;
    cic_bypass_         = NULL;

    capture_rf_ = NULL;
    capture_iq_ = NULL;
    debug_capture_after_mixer_ = NULL;
    debug_capture_after_cic_ = NULL;
    debug_capture_audio_after_ddc_ = NULL;
    debug_capture_audio_after_scaling_ = NULL;
    debug_capture_audio_after_asrc_ = NULL;
    debug_capture_audio_after_weaver_ = NULL;

    debug_capture_debug_dual_ = NULL;
    debug_capture_debug_ = NULL;

    impulse_stimulator_ = NULL;

    debug_fft_real_     = new fft_avg_real(debug_data_size_, fftw_base::hann);
    debug_fft_complex_  = new fft_avg_complex(debug_data_size_, fftw_base::hann);

    debug_time_curve_i_ = new QwtPlotCurve();
    debug_time_curve_i_ -> setPen(QPen(Qt::blue, 2));
    debug_time_curve_i_ -> attach(ui->qwtPlot_debug);

    debug_time_curve_q_ = new QwtPlotCurve();
    debug_time_curve_q_ -> setPen(QPen(Qt::green, 2));
    debug_time_curve_q_ -> attach(ui->qwtPlot_debug);

    debug_freq_curve_ = new QwtPlotCurve();
    debug_freq_curve_ -> setPen(QPen(Qt::blue, 2));
    debug_freq_curve_ -> attach(ui->qwtPlot_debug2);
}

DialogDebugMonitor::~DialogDebugMonitor()
{
    if (local_oscillator_) delete local_oscillator_;
    if (cic_filter_) delete cic_filter_;
    if (cic_bypass_) delete cic_bypass_;

    if (capture_rf_) delete capture_rf_;
    if (capture_iq_) delete capture_iq_;
    if (debug_capture_after_mixer_) delete debug_capture_after_mixer_;
    if (debug_capture_after_cic_) delete debug_capture_after_cic_;
    if (debug_capture_audio_after_ddc_) delete debug_capture_audio_after_ddc_;
    if (debug_capture_audio_after_scaling_) delete debug_capture_audio_after_scaling_;
    if (debug_capture_audio_after_asrc_) delete debug_capture_audio_after_asrc_;
    if (debug_capture_audio_after_weaver_) delete debug_capture_audio_after_weaver_;

    if (impulse_stimulator_) delete impulse_stimulator_;

    if (debug_capture_debug_dual_) delete debug_capture_debug_dual_;
    if (debug_capture_debug_) delete debug_capture_debug_;

    delete debug_time_curve_i_;
    delete debug_time_curve_q_;
    delete debug_freq_curve_;

    delete debug_fft_real_;
    delete debug_fft_complex_;

    delete ui;
}

void DialogDebugMonitor::init() {
    if (capture_rf_) delete capture_rf_;
    capture_rf_ = new Data_Capture(CAPTURE_RF_ADDRESS, CAPTURE_RF_MEMSIZE, CAPTURE_RF_SIZE);

    if (capture_iq_) delete capture_iq_;
    capture_iq_ = new Data_Capture_Dual(CAPTURE_IQ_ADDRESS, CAPTURE_IQ_MEMSIZE, CAPTURE_IQ_SIZE);


    if (debug_capture_after_mixer_) delete debug_capture_after_mixer_;
    debug_capture_after_mixer_= new Data_Capture_Dual(CAPTURE_MIXER_ADDRESS, CAPTURE_MIXER_MEMSIZE, CAPTURE_MIXER_SIZE);

    if (debug_capture_after_cic_) delete debug_capture_after_cic_;
    debug_capture_after_cic_= new Data_Capture_Dual(CAPTURE_CIC_ADDRESS, CAPTURE_CIC_MEMSIZE, CAPTURE_CIC_SIZE);

    if (debug_capture_audio_after_ddc_) delete debug_capture_audio_after_ddc_;
    debug_capture_audio_after_ddc_= new Data_Capture_Dual(CAPTURE_DBG_AUDIO_DDC_ADDRESS, CAPTURE_DBG_AUDIO_DDC_MEMSIZE, CAPTURE_DBG_AUDIO_DDC_SIZE);

    if (debug_capture_audio_after_scaling_) delete debug_capture_audio_after_scaling_;
    debug_capture_audio_after_scaling_= new Data_Capture_Dual(CAPTURE_DBG_AUDIO_SCALING_ADDRESS, CAPTURE_DBG_AUDIO_SCALING_MEMSIZE, CAPTURE_DBG_AUDIO_SCALING_SIZE);

    if (debug_capture_audio_after_asrc_) delete debug_capture_audio_after_asrc_;
    debug_capture_audio_after_asrc_= new Data_Capture_Dual(CAPTURE_DBG_AUDIO_ASRC_ADDRESS, CAPTURE_DBG_AUDIO_ASRC_MEMSIZE, CAPTURE_DBG_AUDIO_ASRC_SIZE);

    if (debug_capture_audio_after_weaver_) delete debug_capture_audio_after_weaver_;
    debug_capture_audio_after_weaver_= new Data_Capture(CAPTURE_DBG_AUDIO_WEAVER_ADDRESS, CAPTURE_DBG_AUDIO_WEAVER_MEMSIZE, CAPTURE_DBG_AUDIO_WEAVER_SIZE);


    if (debug_capture_debug_dual_) delete debug_capture_debug_dual_;
    debug_capture_debug_dual_= new Data_Capture_Dual(CAPTURE_DBG_DBG_DUAL_ADDRESS, CAPTURE_DBG_DBG_DUAL_MEMSIZE, CAPTURE_DBG_DBG_DUAL_SIZE);

    if (debug_capture_debug_) delete debug_capture_debug_;
    debug_capture_debug_= new Data_Capture_Dual(CAPTURE_DBG_DBG_ADDRESS, CAPTURE_DBG_DBG_MEMSIZE, CAPTURE_DBG_DBG_SIZE);


    if (local_oscillator_) delete local_oscillator_;
    local_oscillator_ = new DDS(DDS_LO_ADDRESS, DDS_LO_MEMSIZE, DDS_LO_MASTERFREQUENCY, DDS_LO_PHASEWIDTH);
    on_lo_frequency_lineEdit_editingFinished();

    if (cic_filter_) delete cic_filter_;
    cic_filter_ = new CIC(CIC_ADDRESS, CIC_MEMSIZE, 8);

    if (cic_bypass_) delete cic_bypass_;
    cic_bypass_ = new IP_Driver(CIC_BYPASS_ADDRESS, CIC_BYPASS_MEMSIZE);
    on_comboBox_cic_select_currentIndexChanged(0);

    if (impulse_stimulator_) delete impulse_stimulator_;
    impulse_stimulator_ = new IP_Driver(IMPULSE_STIMU_ADDRESS, IMPULSE_STIMU_MEMSIZE);
    impulse_stimulator_->write(0x0,0);
    impulse_stimulator_->write(0x4,0);

    QTimer *debug_timer = new QTimer(this);
    connect(debug_timer, SIGNAL(timeout()), this, SLOT(run()));
    debug_timer->start(1000); // 1000ms timer
}




void DialogDebugMonitor::run() {
    // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    // Debug Mode

    // 1. capture data according to comboBox_display_select for display

    double debug_signal_i[debug_data_size_];
    double debug_signal_q[debug_data_size_];
    double debug_signal_real[debug_data_size_];

    bool real_signal = false;

    if ((ui->comboBox_display_select->currentIndex() == 0) || (ui->comboBox_display_select->currentIndex() == 7) || (ui->comboBox_display_select->currentIndex() == 9))
        real_signal = true;
    else
        real_signal = false;

    switch (ui->comboBox_display_select->currentIndex()) {
        case (0): capture_rf_ -> capture_and_read(debug_signal_real, debug_data_size_); break;
        case (1): debug_capture_after_mixer_ -> capture_and_read(debug_signal_i, debug_signal_q, debug_data_size_); break;
        case (2): cerr << "err code "<< debug_capture_after_cic_ -> capture_and_read(debug_signal_i, debug_signal_q, debug_data_size_) << endl; break;
        case (3): capture_iq_ -> capture_and_read(debug_signal_i, debug_signal_q, debug_data_size_); break;
        case (4): debug_capture_audio_after_ddc_ -> capture_and_read(debug_signal_i, debug_signal_q, debug_data_size_); break;
        case (5): debug_capture_audio_after_scaling_ -> capture_and_read(debug_signal_i, debug_signal_q, debug_data_size_); break;
        case (6): debug_capture_audio_after_asrc_ -> capture_and_read(debug_signal_i, debug_signal_q, debug_data_size_); break;
        case (7): debug_capture_audio_after_weaver_ -> capture_and_read(debug_signal_real, debug_data_size_); break;
        case (8): debug_capture_debug_dual_ -> capture_and_read(debug_signal_i, debug_signal_q, debug_data_size_); break;
        case (9): debug_capture_debug_ -> capture_and_read(debug_signal_real, debug_data_size_); break;
        default:  cerr << "Debug Mode: Time Domain Debug Display: selected source not available" << endl;
    }


    // 2. display the data in the main waterfall and spectrum display and time domain display

    double debug_time_scale[debug_data_size_];
    for (int i=0; i< debug_data_size_; i++) debug_time_scale[i] = i;

    if (real_signal) {
        debug_time_curve_i_->setData(debug_time_scale, debug_signal_real, debug_data_size_);
        debug_time_curve_q_->setData(debug_time_scale, debug_signal_real, debug_data_size_);
    }
    else {
        debug_time_curve_i_->setData(debug_time_scale, debug_signal_i, debug_data_size_);
        debug_time_curve_q_->setData(debug_time_scale, debug_signal_q, debug_data_size_);
    }

    ui->qwtPlot_debug->replot();


    // 3. do a FFT and display the result (FFT magnitude is not corrected)

    if (real_signal) {
        // RF Data
        double debug_freq_scale[debug_data_size_/2];            // generate x scale
        for (int i=0; i< debug_data_size_/2; i++) debug_freq_scale[i] = i;

        double debug_spectrum[debug_data_size_/2];
        debug_fft_real_ -> reset();
        debug_fft_real_ -> add_fft(debug_signal_real);
        debug_fft_real_ -> get_avg_fft(debug_spectrum);

        debug_freq_curve_ -> setData(debug_freq_scale, debug_spectrum, debug_data_size_/2);
    }
    else {
        // IQ Data
        double debug_freq_scale[debug_data_size_];             // generate x scale
        for (int i=0; i< debug_data_size_; i++) debug_freq_scale[i] = i;

        double debug_spectrum[debug_data_size_];
        debug_fft_complex_ -> reset();
        debug_fft_complex_ -> add_fft(debug_signal_i, debug_signal_q);
        debug_fft_complex_ -> get_avg_fft(debug_spectrum);

        debug_freq_curve_ -> setData(debug_freq_scale, debug_spectrum, debug_data_size_);
    }

    ui->qwtPlot_debug2->replot();

    if (debug_log2file_request_) {

        fstream datei;
        datei.open("debug_log.txt",ios::out);

        if (real_signal) {
            for (int j=0; j<debug_data_size_; j++)
                datei << debug_signal_real[j]<< " ";
        }
        else {
            for (int j=0; j<debug_data_size_; j++)
                datei << debug_signal_i[j]<< " ";

            datei << endl;

            for (int j=0; j<debug_data_size_; j++)
                datei << debug_signal_q[j]<< " ";
        }

        datei.close();

        debug_log2file_request_ = false;
    }

}



void DialogDebugMonitor::set_cic(int decimation_rate) {

    if (decimation_rate <=1) {
        cic_bypass_ -> write(0,0);                        // set cic bypass
        cerr << "debug set CIC to bypassed" << endl;
    }
    else {

        if (decimation_rate <= 512) {
            cic_bypass_ -> write(0,0xFFFFFFF1);                                    // select programmable rate cic and bypass fixed rate cic
            cic_filter_ -> set_rate(decimation_rate);                              // set CIC rate
        }
        else {
            cic_bypass_ -> write(0,0xFFFFFFF3);                                    // select programmable rate cic and fixed rate cic
            cic_filter_ -> set_rate(decimation_rate/64);                           // set CIC rate
        }
        cerr << "set cic to " << decimation_rate << endl;
    }
}

void DialogDebugMonitor::on_comboBox_cic_select_currentIndexChanged(int index)
{
    // cic slot
    int cic_dec = 0;

    switch (index) {
        case 0 : cic_dec = 0; break;
        case 1 : cic_dec = 4; break;
        case 2 : cic_dec = 8; break;
        case 3 : cic_dec = 16; break;
        case 4 : cic_dec = 32; break;
        case 5 : cic_dec = 64; break;
        case 6 : cic_dec = 128; break;
        case 7 : cic_dec = 256; break;
        case 8 : cic_dec = 512; break;
        case 9 : cic_dec = 1024; break;
        case 10 : cic_dec = 2048; break;
        case 11 : cic_dec = 4096; break;
        default:  cic_dec = 0;
    }

    set_cic(cic_dec);
}



void DialogDebugMonitor::on_lo_frequency_lineEdit_editingFinished()
{
    QString req_freq_str = ui -> lo_frequency_lineEdit -> text();    // read frequency from line edit
    double req_freq = req_freq_str.toDouble();
    local_oscillator_ -> set_frequency(req_freq);
}

void DialogDebugMonitor::on_impulse_pushButton_clicked()
{
    impulse_stimulator_ -> write(0x0,1);
    impulse_stimulator_ -> write(0x0,0);
    cerr << "excite impulse" << endl;
}

void DialogDebugMonitor::on_step_pushButton_clicked()
{
    if (impulse_stimulator_ -> read(0x04) == 0) {
        impulse_stimulator_ -> write(0x4,1);
        cerr << "excite step: set RF input to 1" << endl;
    }
    else {
        impulse_stimulator_ -> write(0x4,0);
        cerr << "excite step: set RF input to 0" << endl;
    }
}

void DialogDebugMonitor::on_write2file_pushButton_clicked() {
    debug_log2file_request_ = true;
}
