#ifndef DIALOGDEBUGMONITOR_H
#define DIALOGDEBUGMONITOR_H

#include <fft/fft_avg_complex.h>
#include <fft/fft_avg_real.h>
#include "stipcores.h"
#include <qwt_plot_curve.h>
#include <QDialog>
#include <QTimer>

#include <fstream>

using namespace std;

namespace Ui {
class DialogDebugMonitor;
}

/** @brief Qt Dialog GUI with debug functionalities
  * - reinstanciates all IP cores controlling the waterfall DDC (plus additional debug cores)
  * - uses the macros of the IP core directly from ip_cores_hdmi.h */
class DialogDebugMonitor : public QDialog
{
    Q_OBJECT
    
public:
	/** @brief constructor */
    explicit DialogDebugMonitor(int debug_data_size, QWidget *parent = 0);
	
	/** @brief destructor */
    ~DialogDebugMonitor();

	/** @brief initialization of IP cores using the macro addresses etc. */
    void init();

private slots:
    void run();
    
    void on_comboBox_cic_select_currentIndexChanged(int index);

    void on_lo_frequency_lineEdit_editingFinished();


    void on_impulse_pushButton_clicked();

    void on_step_pushButton_clicked();

    void on_write2file_pushButton_clicked();

private:
    void set_cic(int decimation_rate);

    Ui::DialogDebugMonitor *ui;

    DDS               *local_oscillator_;
    CIC               *cic_filter_;
    IP_Driver         *cic_bypass_;

    Data_Capture      *capture_rf_;
    Data_Capture_Dual *capture_iq_;
    Data_Capture_Dual *debug_capture_after_mixer_;
    Data_Capture_Dual *debug_capture_after_cic_;
    Data_Capture_Dual *debug_capture_audio_after_ddc_;
    Data_Capture_Dual *debug_capture_audio_after_scaling_;
    Data_Capture_Dual *debug_capture_audio_after_asrc_;
    Data_Capture      *debug_capture_audio_after_weaver_;

    Data_Capture_Dual *debug_capture_debug_dual_;
    Data_Capture      *debug_capture_debug_;

    IP_Driver       *impulse_stimulator_;	

    QwtPlotCurve    *debug_time_curve_i_;   // Debug Curves
    QwtPlotCurve    *debug_time_curve_q_;
    QwtPlotCurve    *debug_freq_curve_;

    int             debug_data_size_ ;      // block size for debug cores

    bool            debug_log2file_request_;
    
    fft_avg_real    *debug_fft_real_;		// fft modules
    fft_avg_complex *debug_fft_complex_;

};

#endif // DIALOGDEBUGMONITOR_H
