#ifndef FFT_AVG_COMPLEX_H
#define FFT_AVG_COMPLEX_H

#include "fftw_base.h"

/** @brief  FFT Class for averaging over multiple FFTs with complex time domain input values */
class fft_avg_complex : public fftw_base
{
    int num_ffts_;          // number of added ffts
    double *fft_sum_;       // holds accumulated fft values
    double *fft_out_temp_;  // holds temporary fft output

public:
    /** @brief constructor, allocated memory and calls base class constructor
      * @param n FFT size, number of input values
      * @param fft_window FFT window     */
    fft_avg_complex(int n, t_fft_window fft_window);

    /** @brief destructor */
    ~fft_avg_complex();

    /** @brief performs a single FFT and stores the result internally
      * @param data_in_real[] array of n complex input time domain samples, real part
      * @param data_in_imag[] array of n complex input time domain samples, imaginary part */
    void add_fft(double data_in_real[], double data_in_imag[]);

    /** @brief resets the module, call to restart averaging */
    void reset();

    /** @brief read averaged FFT bins of power spectrum in dBm, including windowing and CPG correction
      * @param output_avg_fft_mag[] array for the n output averaged FFT bins of power spectrum in dBm */
    void get_avg_fft(double output_avg_fft_mag[]);
};

#endif // FFT_AVG_COMPLEX_H
