#ifndef FFT_AVG_REAL_H
#define FFT_AVG_REAL_H

#include "fftw_base.h"

/** @brief  FFT Class for averaging over multiple FFTs with real time domain input values */
class fft_avg_real : public fftw_base
{
    int num_ffts_;          // number of added ffts
    double *fft_sum_;       // holds accumulated fft values
    double *fft_out_temp_;  // holds temporary fft output

public:
    /** @brief constructor, allocated memory and calls base class constructor
      * @param n FFT size, number of input values
      * @param fft_window FFT window     */
    fft_avg_real(int n, t_fft_window fft_window);

    /** @brief destructor */
    ~fft_avg_real();

    /** @brief performs a single FFT and stores the result internally
      * @param data_in[] array of n real input time domain samples */
    void add_fft(double data_in[]);

    /** @brief resets the module, call to restart averaging */
    void reset();

    /** @brief read averaged FFT bins of power spectrum in dBm, including windowing and CPG correction
      * @param output_avg_fft_mag[] array for the n/2 output averaged FFT bins of power spectrum in dBm */
    void get_avg_fft(double output_avg_fft_mag[]);

};

#endif // FFT_AVG_REAL_H
