#ifndef FFT_BASIC_H
#define FFT_BASIC_H

#include "fftw_base.h"

/** @brief Main FFT Class for single non-averaging FFT (performs FFT of real and complex time domain inputs) */
class fft_basic : public fftw_base
{
public:
    /** @brief constructor, calls constructor of base class
      * @param n FFT size & size of input data
      * @param fft_window FFT window */
    fft_basic(int n, t_fft_window fft_window);


    /** @brief performs FFT on real input values, including windowing, with CPG correction
      * @param data_in[] array containing n real input time domain samples
      * @param data_out_mag[] array for the n/2 FFT bins of the power spectrum in dBm  */
    void real_fft_dbm(double data_in[], double data_out_mag[]);

    /** @brief performs FFT on complex input values, including windowing, with CPG correction
      * @param data_in_real[] array containing n input time domain samples, real part
      * @param data_in_imag[] array containing n input time domain samples, imaginary part
      * @param data_out_mag[] array for the n FFT bins of the power spectrum in dBm  */
    void complex_fft_dbm(double data_in_real[], double data_in_imag[], double data_out_mag[]);
};

#endif // FFT_BASIC_H
