#include "frontend.h"

Frontend::Frontend(QObject *parent) : QObject(parent)
{
    scaler_rf_ = NULL;
    relais_controller_ = NULL;

    auto_scaling_ = false;
    manual_scaler_setting_db_ = 0;

}

Frontend::~Frontend()
{
    if (scaler_rf_) delete scaler_rf_;
    if (relais_controller_) delete relais_controller_;
}


void Frontend::init(int signalscaler_adr, int signalscaler_memsize, int signalscaler_max, int relais_controller_adr, int relais_controller_size) {
    if (scaler_rf_) delete scaler_rf_;
    scaler_rf_ = new Signal_Scaler(signalscaler_adr, signalscaler_memsize, signalscaler_max);
    set_rf_scaler(0);

    max_scaling_db_ = signalscaler_max;

    if (relais_controller_) delete relais_controller_;
    relais_controller_ = new IP_Driver(relais_controller_adr, relais_controller_size);
    relais_controller_ -> write(0,0);


    // Timer for RF Scaler Update
    QTimer *timer_rf_scaler = new QTimer(this);
    connect(timer_rf_scaler, SIGNAL(timeout()), this, SLOT(timer_slot()));
    timer_rf_scaler->start(3000);
}



void Frontend::get_scaler_setting(int scaling_value_db) {
    manual_scaler_setting_db_ = scaling_value_db;

    if (!auto_scaling_) set_rf_scaler(manual_scaler_setting_db_);
}

void Frontend::get_auto_setting(bool auto_setting) {
    auto_scaling_ = auto_setting;
    if (!auto_scaling_) set_rf_scaler(manual_scaler_setting_db_);
}


void Frontend::set_rf_scaler(int scaling_value_db) {
    scaler_rf_ -> set_scaling(scaling_value_db);
    emit this -> send_scaling_value(scaling_value_db);
}

void Frontend::get_relais_setting(int value) {

    int relais1_left = 0;
    int relais1_right = 0;
    int relais2_left = 0;
    int relais2_right = 0;

    relais_controller_ -> write(0,0);

    switch (value) {
        case 0: relais1_left = 1; relais2_left = 1;   break;
        case 1: relais1_right = 1; relais2_left = 1;  break;
        case 2: relais1_right = 1; relais2_right = 1; break;
    }

    int write_value = relais1_left + 2*relais1_right +4*relais2_left + 8*relais2_right;
    relais_controller_ -> write(0,write_value);
    sleep(1);
    relais_controller_ -> write(0,0);
}


void Frontend::timer_slot() {
    // get max RF value since last timer step
    int rf_max_value = scaler_rf_ -> get_max_value();
    int rf_max_value_dbfs = (int) (20*log10((double)rf_max_value / 32768));

    emit this -> send_rf_level(rf_max_value, rf_max_value_dbfs);
    scaler_rf_ -> reset_max_value();


    // Scale the signal automatically
	// it may advantageous to implement a hysteresis here, to avoid frequent changes
	// large scaler values may amplify the DC bias

    if (auto_scaling_) {

        if (rf_max_value_dbfs > -10) {
            // set to no amplification
            set_rf_scaler(0);
        }

        if (rf_max_value_dbfs < -20) {
            // set amplification
            int requested_scaling_db = ((-rf_max_value_dbfs-10) / 6 )*6; // round to multiple of 6
            if (requested_scaling_db <= max_scaling_db_)
                set_rf_scaler(requested_scaling_db );
            else
                set_rf_scaler(max_scaling_db_);
        }
    }

}
