#ifndef IP_CORES_H
#define IP_CORES_H

/**  Addresses and memory sizes (in k) of IP cores used in the Radio Design  (read from Vivado) */


// --------------------------------------------------------------------------------
// Frontend
// --------------------------------------------------------------------------------
#define ADC_SPI_ADDRESS             0x43C70000
#define ADC_SPI_MEMSIZE             64

#define ADC_DATA_ADDRESS            0x43C30000
#define ADC_DATA_MEMSIZE            64

#define SIGNALSCALER_RF_ADDRESS     0x43C90000
#define SIGNALSCALER_RF_MEMSIZE     64
//#define SIGNALSCALER_RF_MAX         4      // value specified in ip core (shift value bw)
#define SIGNALSCALER_RF_MAX_DB      48      // max scaling value in db

#define INPUT_MUX_ADDRESS           0x41250000
#define INPUT_MUX_MEMSIZE           64

#define RELAIS_CTRL_ADDRESS         0x00
#define RELAIS_CTRL_SIZE            64

// --------------------------------------------------------------------------------
// Waterfall DDC
// --------------------------------------------------------------------------------

#define CAPTURE_RF_ADDRESS          0x43C20000
#define CAPTURE_RF_MEMSIZE          64
#define CAPTURE_RF_SIZE             4096  // min 4096

#define IQ_RESET_ADDRESS            0x41210000
#define IQ_RESET_MEMSIZE            64

#define MIXER_GAIN                  26          // aus Matlab


// DDC
#define DDS_LO_ADDRESS          0x43C60000
#define DDS_LO_MEMSIZE          64
#define DDS_LO_MASTERFREQUENCY  250000     // clock (master) clock frequency of the core in kHz
#define DDS_LO_PHASEWIDTH       28         // phase width (look up in vivado under block design -> DDS menu -> Summary -> Phase Width (not Phase Angle Width))

#define CIC_ADDRESS             0x43C40000
#define CIC_MEMSIZE             64
#define CIC_GAIN                0

#define CIC_BYPASS_ADDRESS      0x41200000
#define CIC_BYPASS_MEMSIZE      64

#define FIR_GAIN                5       // einfach gemessen

#define CAPTURE_IQ_ADDRESS      0x43C10000
#define CAPTURE_IQ_MEMSIZE      64
#define CAPTURE_IQ_SIZE         8192 //16384



// --------------------------------------------------------------------------------
// Audio RX
// --------------------------------------------------------------------------------

#define AUDIO_RX_RESET_IQ_ADDRESS       0x41220000
#define AUDIO_RX_RESET_IQ_MEMSIZE       64

#define AUDIO_RX_DDC_SELECT_ADDRESS     0x41240000
#define AUDIO_RX_DDC_SELECT_MEMSIZE     64

#define AUDIO_RX_OUT_CAPTURE_IQ_ADDRESS 0x43CA0000
#define AUDIO_RX_OUT_CAPTURE_IQ_MEMSIZE 64
#define AUDIO_RX_OUT_CAPTURE_IQ_SIZE    256

// Audio DDC 0
#define AUDIO_DDC0_LO_ADDRESS           0x43CF0000
#define AUDIO_DDC0_LO_MEMSIZE           64
#define AUDIO_DDC0_LO_MASTERFREQUENCY   250000
#define AUDIO_DDC0_LO_PHASEWIDTH        28

#define AUDIO_DDC0_CAPTURE_IQ_ADDRESS   0x43D40000
#define AUDIO_DDC0_CAPTURE_IQ_MEMSIZE   64
#define AUDIO_DDC0_CAPTURE_IQ_SIZE      256

// Audio DDC 1
#define AUDIO_DDC1_LO_ADDRESS           0x43D90000
#define AUDIO_DDC1_LO_MEMSIZE           64
#define AUDIO_DDC1_LO_MASTERFREQUENCY   250000
#define AUDIO_DDC1_LO_PHASEWIDTH        28

#define AUDIO_DDC1_CAPTURE_IQ_ADDRESS   0x43D70000
#define AUDIO_DDC1_CAPTURE_IQ_MEMSIZE   64
#define AUDIO_DDC1_CAPTURE_IQ_SIZE      256


// Audio Backend
#define AUDIO_RX_SCALER_I_ADDRESS       0x43CE0000
#define AUDIO_RX_SCALER_I_MEMSIZE       64

#define AUDIO_RX_SCALER_Q_ADDRESS       0x43D00000
#define AUDIO_RX_SCALER_Q_MEMSIZE       64

//#define AUDIO_RX_SCALER_MAX             4
#define AUDIO_RX_SCALER_MAX_DB          48

#define AUDIO_RX_MUX_CTRL_ADDRESS       0x41230000
#define AUDIO_RX_MUX_CTRL_MEMSIZE       64

#define AUDIO_WEAVER_LO_ADDRESS         0x43D50000
#define AUDIO_WEAVER_LO_MEMSIZE         64
#define AUDIO_WEAVER_LO_MASTERFREQUENCY 100000     // clock (master) clock frequency of the core in kHz
#define AUDIO_WEAVER_LO_PHASEWIDTH      27         // phase width (look up in vivado under block design -> DDS menu -> Summary -> Phase Width (not Phase Angle Width))




// --------------------------------------------------------------------------------
// Debug Cores
// --------------------------------------------------------------------------------

// RF Testgenerator
#define DDS_TESTRF_ADDRESS                  0x43C50000
#define DDS_TESTRF_MEMSIZE                  64
#define DDS_TESTRF_MASTERFREQUENCY          250000     // clock (master) clock frequency of the core in Hz
#define DDS_TESTRF_PHASEWIDTH               28                // phase width (look up in vivado under block design -> DDS menu -> Summary -> Phase Width (not Phase Angle Width))

// Impulse Stimulator
#define IMPULSE_STIMU_ADDRESS               0x43D80000
#define IMPULSE_STIMU_MEMSIZE               64

// Waterfall DDC
#define CAPTURE_MIXER_ADDRESS               0x43CD0000
#define CAPTURE_MIXER_MEMSIZE               64
#define CAPTURE_MIXER_SIZE                  512

#define CAPTURE_CIC_ADDRESS                 0x43CC0000
#define CAPTURE_CIC_MEMSIZE                 64
#define CAPTURE_CIC_SIZE                    512

//audio RX
#define CAPTURE_DBG_AUDIO_DDC_ADDRESS       0x43D40000
#define CAPTURE_DBG_AUDIO_DDC_MEMSIZE       64
#define CAPTURE_DBG_AUDIO_DDC_SIZE          512

#define CAPTURE_DBG_AUDIO_SCALING_ADDRESS   0x43CB0000
#define CAPTURE_DBG_AUDIO_SCALING_MEMSIZE   64
#define CAPTURE_DBG_AUDIO_SCALING_SIZE      512

#define CAPTURE_DBG_AUDIO_ASRC_ADDRESS      0x43CA0000
#define CAPTURE_DBG_AUDIO_ASRC_MEMSIZE      64
#define CAPTURE_DBG_AUDIO_ASRC_SIZE         512

#define CAPTURE_DBG_AUDIO_WEAVER_ADDRESS    0x43D10000
#define CAPTURE_DBG_AUDIO_WEAVER_MEMSIZE    64
#define CAPTURE_DBG_AUDIO_WEAVER_SIZE       512


// General Purpose, Complex/Dual
#define CAPTURE_DBG_DBG_DUAL_ADDRESS        0x43D20000 // 43D20000 = weaver LO
#define CAPTURE_DBG_DBG_DUAL_MEMSIZE        64
#define CAPTURE_DBG_DBG_DUAL_SIZE           512

// General Purpose, Real
#define CAPTURE_DBG_DBG_ADDRESS             0x43C20000 // dummy RF capture
#define CAPTURE_DBG_DBG_MEMSIZE             64
#define CAPTURE_DBG_DBG_SIZE                512


#define AUDIO_RX_USE_FIFO 0 //?

#endif // IP_CORES_H
