#include "mainwindow.h"
#include "ui_mainwindow.h"

//#include "ip_cores.h"
#include "ip_cores_hdmi.h"

#include "../../stipcores/header/stipcores.h"
#include <fftw3.h>
#include "qwt_thermo.h"
#include <fstream>
#include <QtGui>


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui_debug_monitor = new DialogDebugMonitor(512);

    // Frontend
    rx_frontend_ = NULL;

    ui -> adcmax_Thermo -> setRange(-70, 0);
    ui -> scaler_low_label->setText(QString::number(-SIGNALSCALER_RF_MAX_DB, 10));
    ui -> scaler_high_label->setText("+" + QString::number(SIGNALSCALER_RF_MAX_DB, 10));
    ui -> rfscaling_horizontalSlider->setMinimum((int)-SIGNALSCALER_RF_MAX_DB/6);
    ui -> rfscaling_horizontalSlider->setMaximum((int)SIGNALSCALER_RF_MAX_DB/6);


    // Debug Stuff
    testgenerator_ = NULL;

    // Audio Stuff
    audio_rx_ = NULL;

    ui -> audiomax_Thermo -> setRange(-100, 0);
    ui -> audio_scaler_low_label->setText(QString::number(-AUDIO_RX_SCALER_MAX_DB, 10));
    ui -> audio_scaler_high_label->setText("+" + QString::number(AUDIO_RX_SCALER_MAX_DB, 10));
    ui -> audioscaling_horizontalSlider->setMinimum((int)-AUDIO_RX_SCALER_MAX_DB/6);
    ui -> audioscaling_horizontalSlider->setMaximum((int)AUDIO_RX_SCALER_MAX_DB/6);

    ui->audio_narrow_bar_0_widget->setStyleSheet("background-color:rgba(255,255,255,20%)");
    ui->audio_medium_bar_0_widget->setStyleSheet("background-color:rgba(255,255,255,20%)");
    ui->audio_wide_bar_0_widget->setStyleSheet("background-color:rgba(255,255,255,20%)");
    ui->audio_narrow_bar_1_widget->setStyleSheet("background-color:rgba(255,255,255,20%)");
    ui->audio_medium_bar_1_widget->setStyleSheet("background-color:rgba(255,255,255,20%)");
    ui->audio_wide_bar_1_widget->setStyleSheet("background-color:rgba(255,255,255,20%)");

    ui->audio_narrow_bar_0_widget->setVisible(false);
    ui->audio_medium_bar_0_widget->setVisible(false);
    ui->audio_narrow_bar_1_widget->setVisible(false);
    ui->audio_medium_bar_1_widget->setVisible(false);
    ui->audio_wide_bar_1_widget->setVisible(false);

    adc = NULL;
    iq_reset = NULL;

    // Waterfall RX
    waterfall_ddc_ = NULL;
    waterfall_1a_ddc_ = NULL;
    waterfall_1b_ddc_ = NULL;

    power_level_min_ = -140;	// determined minimum and maximum power level for the waterfall displays
    power_level_max_ = 10;

    // set up status bar
    statusBar()->showMessage("Radio not yet connected");

    debug_monitor_activated_ = false;
    currently_recording_ = false;

    timer_ = NULL;
}

MainWindow::~MainWindow()
{
    if (rx_frontend_) delete rx_frontend_;
    if (testgenerator_) delete testgenerator_;
    if (audio_rx_) delete audio_rx_;
    if (adc) delete adc;
    if (iq_reset) delete iq_reset;
    if (waterfall_ddc_) delete waterfall_ddc_;
    if (waterfall_1a_ddc_) delete waterfall_1a_ddc_;
    if (waterfall_1b_ddc_) delete waterfall_1b_ddc_;
    if (timer_) delete timer_;

    delete ui_debug_monitor;
    delete ui;
}


void MainWindow::on_connect_pushButton_clicked()
{
    cerr << "==================================================================================================================================" << endl;
    cerr << "  connect IP cores & initialize the radio" << endl;
    cerr << "==================================================================================================================================" << endl << endl;


    // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    // initialize frontend
    // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    if (rx_frontend_) delete rx_frontend_;
    rx_frontend_ = new Frontend();
    rx_frontend_ -> init(SIGNALSCALER_RF_ADDRESS, SIGNALSCALER_RF_MEMSIZE, SIGNALSCALER_RF_MAX_DB, RELAIS_CTRL_ADDRESS, RELAIS_CTRL_SIZE);

    connect(this, SIGNAL(send_frontend_auto_setting(bool)), rx_frontend_, SLOT(get_auto_setting(bool)));
    connect(this, SIGNAL(send_frontend_scaler_setting(int)), rx_frontend_, SLOT(get_scaler_setting(int)));
    connect(ui->input_source_comboBox, SIGNAL(currentIndexChanged(int)), rx_frontend_, SLOT(get_relais_setting(int)));
    connect(rx_frontend_, SIGNAL(send_rf_level(int,int)), this, SLOT(get_frontend_rf_value(int, int)));
    connect(rx_frontend_, SIGNAL(send_scaling_value(int)), this, SLOT(get_frontend_scaling_value(int)));

    // inits
    rx_frontend_->get_auto_setting(ui->checkBox_rf_scale_auto->isChecked());
    rx_frontend_->get_scaler_setting(ui->rfscaling_horizontalSlider->sliderPosition() * 6);
    rx_frontend_->get_relais_setting(0); //init input relais to input 0;

    // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    // initialize debug
    // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    if (testgenerator_) delete testgenerator_;
    testgenerator_ = new Testgenerator();
    testgenerator_ -> init_input_mux(INPUT_MUX_ADDRESS, INPUT_MUX_MEMSIZE, 0);
    testgenerator_ -> init_generator(DDS_TESTRF_ADDRESS, DDS_TESTRF_MEMSIZE, DDS_TESTRF_MASTERFREQUENCY, DDS_TESTRF_PHASEWIDTH);

    // internal
    connect(ui->signal_source_comboBox, SIGNAL(currentIndexChanged(int)), testgenerator_, SLOT(get_signal_source(int)));
    connect(ui->radioButton_test_gen_single, SIGNAL(clicked(bool)), this, SLOT(debug_single_selected()));
    connect(ui->rftest_frequency_lineEdit, SIGNAL(editingFinished()), this, SLOT(debug_single_selected()));
    connect(ui->radioButton_test_gen_sweep_, SIGNAL(clicked(bool)), this, SLOT(debug_sweep_selected()));
    connect(ui->rftest_start_freq_lineEdit, SIGNAL(editingFinished()), this, SLOT(debug_sweep_selected()));
    connect(ui->rftest_stop_freq_lineEdit, SIGNAL(editingFinished()), this, SLOT(debug_sweep_selected()));
    connect(ui->rftest_freq_step_lineEdit, SIGNAL(editingFinished()), this, SLOT(debug_sweep_selected()));
    connect(ui->sweep_stop_CheckBox, SIGNAL(clicked(bool)), testgenerator_, SLOT(get_sweep_stopped(bool)));

    // to testgenerator class
    connect(testgenerator_, SIGNAL(send_true_frequency(double)), this, SLOT(get_debug_true_frequency(double)));
    connect(this, SIGNAL(send_debug_single(double)), testgenerator_, SLOT(get_single(double)));
    connect(this, SIGNAL(send_debug_sweep(double,double,double)), testgenerator_, SLOT(get_sweep(double, double, double)));

    // inits
    testgenerator_ ->get_sweep_stopped( ui->sweep_stop_CheckBox->isChecked());
    debug_sweep_selected();
    debug_single_selected();
    testgenerator_-> get_signal_source(ui->signal_source_comboBox->currentIndex());


    // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    // initialize Audio RX
    // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    if (audio_rx_) delete audio_rx_;
    audio_rx_ = new AudioRX();

    audio_rx_ -> init_reset(AUDIO_RX_RESET_IQ_ADDRESS, AUDIO_RX_RESET_IQ_MEMSIZE);
    audio_rx_ -> init_ddc_selector(AUDIO_RX_DDC_SELECT_ADDRESS, AUDIO_RX_DDC_SELECT_MEMSIZE);
    audio_rx_ -> init_output_spectrum_display(AUDIO_RX_OUT_CAPTURE_IQ_ADDRESS, AUDIO_RX_OUT_CAPTURE_IQ_MEMSIZE, AUDIO_RX_OUT_CAPTURE_IQ_SIZE, ui->audio_out_qwtPlot);
    audio_rx_ -> init_scaling(AUDIO_RX_SCALER_I_ADDRESS, AUDIO_RX_SCALER_I_MEMSIZE, AUDIO_RX_SCALER_Q_ADDRESS, AUDIO_RX_SCALER_Q_MEMSIZE, AUDIO_RX_SCALER_MAX_DB);
    audio_rx_ -> init_mux_controller(AUDIO_RX_MUX_CTRL_ADDRESS, AUDIO_RX_MUX_CTRL_MEMSIZE);
    audio_rx_ -> init_weaver_lo(AUDIO_WEAVER_LO_ADDRESS, AUDIO_WEAVER_LO_MEMSIZE, AUDIO_WEAVER_LO_MASTERFREQUENCY, AUDIO_WEAVER_LO_PHASEWIDTH, 12);

    // DDC 0
    audio_rx_ -> init_lo(AUDIO_DDC0_LO_ADDRESS, AUDIO_DDC0_LO_MEMSIZE, AUDIO_DDC0_LO_MASTERFREQUENCY, AUDIO_DDC0_LO_PHASEWIDTH, 10000, 0);
    audio_rx_ -> init_capture(AUDIO_DDC0_CAPTURE_IQ_ADDRESS, AUDIO_DDC0_CAPTURE_IQ_MEMSIZE, AUDIO_DDC0_CAPTURE_IQ_SIZE, 0);
    audio_rx_ -> init_waterfall(ui->waterfall_audio_ddc0, 125000, power_level_min_, power_level_max_, 30.5, AUDIO_DDC0_CAPTURE_IQ_SIZE, 80, 0);

    // DDC 1
    audio_rx_ -> init_lo(AUDIO_DDC1_LO_ADDRESS, AUDIO_DDC1_LO_MEMSIZE, AUDIO_DDC1_LO_MASTERFREQUENCY, AUDIO_DDC1_LO_PHASEWIDTH, 10000, 1);
    audio_rx_ -> init_capture(AUDIO_DDC1_CAPTURE_IQ_ADDRESS, AUDIO_DDC1_CAPTURE_IQ_MEMSIZE, AUDIO_DDC1_CAPTURE_IQ_SIZE, 1);
    audio_rx_ -> init_waterfall(ui->waterfall_audio_ddc1, 125000, power_level_min_, power_level_max_, 30.5, AUDIO_DDC1_CAPTURE_IQ_SIZE, 80, 1);

    // connect scaler
    connect(audio_rx_, SIGNAL(send_audio_out_level(int,int)), this, SLOT(get_audio_out_level(int,int)));
    connect(audio_rx_, SIGNAL(send_scaling_value(int)), this, SLOT(get_audio_scaling_value(int)));

    // connect audio properties
    connect(ui->wideband_radioButton, SIGNAL(clicked(bool)), this, SLOT(get_audio_properties()));
    connect(ui->mediumband_radioButton, SIGNAL(clicked(bool)), this, SLOT(get_audio_properties()));
    connect(ui->narrowband_radioButton, SIGNAL(clicked(bool)), this, SLOT(get_audio_properties()));
    connect(ui->invert_spectrum_checkBox, SIGNAL(clicked(bool)), this, SLOT(get_audio_properties()));
    connect(ui->binaural_checkBox, SIGNAL(clicked(bool)), this, SLOT(get_audio_properties()));
    connect(this, SIGNAL(send_audio_output_properties(int,bool,bool)), audio_rx_, SLOT(get_output_properties(int,bool,bool)));
    connect(this, SIGNAL(send_audio_ddc_selection(int,bool)), audio_rx_, SLOT(get_output_ddc_selection(int,bool)));
    connect(this, SIGNAL(send_audio_rx_frequency(double,int)), audio_rx_, SLOT(get_ddc_frequency(double,int)));
    connect(this, SIGNAL(send_audio_output_scaler_setting(int)), audio_rx_, SLOT(get_output_scaler_setting(int)));
    connect(this, SIGNAL(send_audio_output_scaler_auto_setting(bool)), audio_rx_, SLOT(get_output_scaler_auto_setting(bool)));
    connect(audio_rx_,SIGNAL(send_new_frequency_ddc0(double)), this, SLOT(get_audio_ddc0_new_frequency(double)));
    connect(audio_rx_,SIGNAL(send_new_frequency_ddc1(double)), this, SLOT(get_audio_ddc1_new_frequency(double)));
    connect(ui->audio_out_checkBox, SIGNAL(clicked(bool)), audio_rx_, SLOT(get_audio_headphone_out(bool)));

    //init
    emit this->send_audio_output_scaler_auto_setting(ui->checkBox_audio_scale_auto->isChecked());
    emit this -> send_audio_output_scaler_setting(ui->audioscaling_horizontalSlider->sliderPosition() * 6);
    audio_rx_ -> reset();
    audio_rx_ -> correct_size();
    get_audio_properties();
    on_audio_ddc0_freq_lineEdit_textChanged();
    on_audio_ddc1_freq_lineEdit_textChanged();
    emit this->send_audio_ddc_selection(0,ui->select_audio_ddc0_Button->isChecked());
    emit this->send_audio_ddc_selection(1,ui->select_audio_ddc1_Button->isChecked());



    cerr << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"<<endl;
    cerr << " setup ADC, Testgenerator, Scaler RF, Capture RF and Debug IP Cores"<<endl;
    cerr << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"<<endl<<endl;

    if (adc) delete adc;
    adc = new AD9467(ADC_SPI_ADDRESS, ADC_SPI_MEMSIZE, ADC_DATA_ADDRESS, ADC_DATA_MEMSIZE);
    adc -> init();

    if (iq_reset) delete iq_reset;
    iq_reset = new IP_Driver(IQ_RESET_ADDRESS, IQ_RESET_MEMSIZE);
    iq_reset->write(0,0xFFFFFFFF);

    // set gains of mixers and filters
    gain_mixer = MIXER_GAIN;
    gain_cic = CIC_GAIN;
    gain_fir = FIR_GAIN;
    gain_fir_audio = 5;      // empirically


    double sample_frequency = 250000; 				// in kHz
    double usable_span_frac = 0.8;					// aliasing free fraction of the spectrum
    double max_frequency = (sample_frequency/2);


    // Waterfall RX
    cerr << endl;
    cerr << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"<<endl;
    cerr << " setup Waterfall RX "<<endl;
    cerr << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"<<endl<<endl;

    if (waterfall_ddc_) delete waterfall_ddc_;
    waterfall_ddc_ = new WaterfallDDC();

    waterfall_ddc_ -> init_lo(DDS_LO_ADDRESS, DDS_LO_MEMSIZE, DDS_LO_MASTERFREQUENCY, DDS_LO_PHASEWIDTH, 10000);
    waterfall_ddc_ -> init_cic(CIC_ADDRESS, CIC_MEMSIZE, CIC_BYPASS_ADDRESS, CIC_BYPASS_MEMSIZE);
    waterfall_ddc_ -> init_iq_capture(CAPTURE_IQ_ADDRESS, CAPTURE_IQ_MEMSIZE, CAPTURE_IQ_SIZE);
    waterfall_ddc_ -> init_rf_capture(CAPTURE_RF_ADDRESS, CAPTURE_RF_MEMSIZE, CAPTURE_RF_SIZE);
    waterfall_ddc_ -> init_waterfall(ui->waterfall_rf, max_frequency, power_level_min_, power_level_max_, max_frequency, 1024, 200, usable_span_frac);
    waterfall_ddc_ -> init_fft(CAPTURE_RF_SIZE, CAPTURE_IQ_SIZE, 1);
    waterfall_ddc_ -> init_labels(ui->zoom_label, ui->resolution_label, ui->bandwidth_label, ui->lo_freq_label, ui->cic_dec_label, ui->fir_dec_label, ui->fft_label);
    waterfall_ddc_ -> init_spectrum_line_plot(ui->qwtPlot_main_spectrum);

    waterfall_ddc_ -> correct_waterfall_size();

    // small Waterfall RX 1a
    cerr << endl;
    cerr << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"<<endl;
    cerr << " setup small Waterfall RX 1a "<<endl;
    cerr << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"<<endl<<endl;

    if (waterfall_1a_ddc_) delete waterfall_1a_ddc_;
    waterfall_1a_ddc_ = new WaterfallDDC();

    waterfall_1a_ddc_ -> init_lo(DDS_LO_ADDRESS, DDS_LO_MEMSIZE, DDS_LO_MASTERFREQUENCY, DDS_LO_PHASEWIDTH, 10000);
    waterfall_1a_ddc_ -> init_cic(CIC_ADDRESS, CIC_MEMSIZE, CIC_BYPASS_ADDRESS, CIC_BYPASS_MEMSIZE);
    waterfall_1a_ddc_ -> init_iq_capture(CAPTURE_IQ_ADDRESS, CAPTURE_IQ_MEMSIZE, CAPTURE_IQ_SIZE);
    waterfall_1a_ddc_ -> init_rf_capture(CAPTURE_RF_ADDRESS, CAPTURE_RF_MEMSIZE, CAPTURE_RF_SIZE);
    waterfall_1a_ddc_ -> init_waterfall(ui->waterfall_rf_1a, max_frequency, power_level_min_, power_level_max_, max_frequency, 512, 120, usable_span_frac);
    waterfall_1a_ddc_ -> init_fft(CAPTURE_RF_SIZE, CAPTURE_IQ_SIZE, 1);

    waterfall_1a_ddc_ -> correct_waterfall_size();


    // small Waterfall RX 1b
    cerr << endl;
    cerr << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"<<endl;
    cerr << " setup small Waterfall RX 1b"<<endl;
    cerr << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"<<endl<<endl;

    if (waterfall_1b_ddc_) delete waterfall_1b_ddc_;
    waterfall_1b_ddc_ = new WaterfallDDC();

    waterfall_1b_ddc_ -> init_lo(DDS_LO_ADDRESS, DDS_LO_MEMSIZE, DDS_LO_MASTERFREQUENCY, DDS_LO_PHASEWIDTH, 10000);
    waterfall_1b_ddc_ -> init_cic(CIC_ADDRESS, CIC_MEMSIZE, CIC_BYPASS_ADDRESS, CIC_BYPASS_MEMSIZE);
    waterfall_1b_ddc_ -> init_iq_capture(CAPTURE_IQ_ADDRESS, CAPTURE_IQ_MEMSIZE, CAPTURE_IQ_SIZE);
    waterfall_1b_ddc_ -> init_rf_capture(CAPTURE_RF_ADDRESS, CAPTURE_RF_MEMSIZE, CAPTURE_RF_SIZE);
    waterfall_1b_ddc_ -> init_waterfall(ui->waterfall_rf_1b, max_frequency, power_level_min_, power_level_max_, max_frequency, 512, 120, usable_span_frac);
    waterfall_1b_ddc_ -> init_fft(CAPTURE_RF_SIZE, CAPTURE_IQ_SIZE, 1);

    waterfall_1b_ddc_ -> correct_waterfall_size();


    // connects signals from (panorama) waterfall plots, that handle audio frequency selection by clicking
    connect(waterfall_ddc_, SIGNAL(audio_frequency_clicked(double)), this, SLOT(get_audio_frequency_clicked(double)));
    connect(waterfall_1a_ddc_, SIGNAL(audio_frequency_clicked(double)), this, SLOT(get_audio_frequency_clicked(double)));
    connect(waterfall_1b_ddc_, SIGNAL(audio_frequency_clicked(double)), this, SLOT(get_audio_frequency_clicked(double)));


    // Color Map Update for Waterfalls
    connect(ui->colormap_main_ScrollBar, SIGNAL(sliderMoved(int)), this, SLOT(update_waterfall_color_map()));
    connect(ui->colormap_left_ScrollBar, SIGNAL(sliderMoved(int)), this, SLOT(update_waterfall_color_map()));
    connect(ui->colormap_right_ScrollBar, SIGNAL(sliderMoved(int)), this, SLOT(update_waterfall_color_map()));
    connect(ui->colormap_mode_checkBox, SIGNAL(clicked(bool)), this, SLOT(update_waterfall_color_map()));
    update_waterfall_color_map();

    // Timer for Waterfall Update
    if (timer_) delete timer_;
    timer_ = new QTimer(this);
    connect(timer_, SIGNAL(timeout()), this, SLOT(update_waterfall()));
    on_update_rate_comboBox_currentIndexChanged(ui->update_rate_comboBox->currentIndex());


    // enable buttons for dialogs
    ui->avg_ffts_comboBox->setEnabled(true);


    // Setup Debug Functionality
    connect(ui->debugmonitor_pushButton, SIGNAL(clicked()), this, SLOT(on_debugmonitor_pushButton_clicked()));
    connect(ui->reset_waterfall_iq_pushButton, SIGNAL(clicked()), this, SLOT(waterfall_reset()));
    connect(ui->reset_audio_iq_pushButton, SIGNAL(clicked), this, SLOT(audio_reset()));
    connect(ui->correct_plot_sizes_pushButton, SIGNAL(clicked()), this, SLOT(correct_plots()));

    // update status bar
    statusBar()->showMessage("Radio successfully connected");

    cerr << "==================================================================================================================================" << endl;
    cerr << "  IP core and radio intialization done" << endl;
    cerr << "==================================================================================================================================" << endl << endl;

}




void MainWindow::update_waterfall()
{
    QTime runtime_measure;
    runtime_measure.start();
    int waterfall_time = 0;
    int waterfall_1a_time = 0;
    int waterfall_1b_time = 0;
    int audio_rx_time = 0;
    int total_time = 0;


    // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //   1. capture data according to zoom_level
    //   2. display the data in the main waterfall and spectrum display
    // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	// Waterfall Displays
    waterfall_ddc_ -> set_gain_rf_scaler(gain_scaler_rf);
    waterfall_ddc_ -> set_gain_ddc(gain_mixer + gain_cic + gain_fir);
    waterfall_ddc_ -> capture_and_show(record_file_, currently_recording_);
    waterfall_time = runtime_measure.elapsed();

    if (ui->waterfall_1a_checkBox->isChecked()) {
        waterfall_1a_ddc_ -> set_gain_rf_scaler(gain_scaler_rf);
        waterfall_1a_ddc_ -> set_gain_ddc(gain_mixer + gain_cic + gain_fir);
        waterfall_1a_ddc_ -> capture_and_show(record_file_, false);
    }
    waterfall_1a_time = runtime_measure.elapsed();

    if (ui->waterfall_1b_checkBox->isChecked()) {
        waterfall_1b_ddc_ -> set_gain_rf_scaler(gain_scaler_rf);
        waterfall_1b_ddc_ -> set_gain_ddc(gain_mixer + gain_cic + gain_fir);
        waterfall_1b_ddc_ -> capture_and_show(record_file_, false);
    }
    waterfall_1b_time = runtime_measure.elapsed();


	// Audio Waterfall Displays
    if (!debug_monitor_activated_) {
        if (ui->select_audio_ddc0_Button->isChecked())
            audio_rx_->capture_and_show(gain_mixer + gain_scaler_rf + gain_cic + gain_fir_audio, 0);

        if (ui->select_audio_ddc1_Button->isChecked())
            audio_rx_->capture_and_show(gain_mixer + gain_scaler_rf + gain_cic + gain_fir_audio, 1);

        if (ui->audio_out_checkBox->isChecked())
            audio_rx_->capture_and_show_spectrum_out();

    }
    audio_rx_time = runtime_measure.elapsed();


    // determine required execution time
    total_time = runtime_measure.elapsed();

    cerr << "Waterfall & Line Plot " << waterfall_time << endl;
    cerr << "Waterfall 1a          " << waterfall_1a_time - waterfall_time << endl;
    cerr << "Waterfall 1b          " << waterfall_1b_time - waterfall_1a_time << endl;
    cerr << "Audio RX              " << audio_rx_time - waterfall_1b_time << endl;
    cerr << "------------------------------------------" << endl;
    cerr << "total time (ms)       " << total_time << endl << endl;

}



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// =====================================================================================================================
//   FRONTEND FUNCTIONALITY
// =====================================================================================================================

// scaler moved -> send value in dB to frontend class
void MainWindow::on_rfscaling_horizontalSlider_sliderMoved(int position) {
    emit this -> send_frontend_scaler_setting(position * 6);
}

// automatic check box clicked -> send value to frontend class and enable or disable slider
void MainWindow::on_checkBox_rf_scale_auto_clicked(bool checked) {

    if (checked)
        ui->rfscaling_horizontalSlider->setDisabled(true);
    else
        ui->rfscaling_horizontalSlider->setDisabled(false);

    emit this -> send_frontend_auto_setting(checked);
}

// new RF level received -> display in thermo widget
void MainWindow::get_frontend_rf_value(int rf_level, int rf_level_dbfs) {
    ui -> adcmax_label -> setText(QString::number(rf_level, 10));
    ui -> adcmax_db_label -> setText("(" + QString::number(rf_level_dbfs, 10) + " dBFS)");

    if (rf_level_dbfs > -3)
        ui->adcmax_Thermo->setFillColor(QColor("red"));
    else
        ui->adcmax_Thermo->setFillColor(QColor("green"));

    ui->adcmax_Thermo->setValue(rf_level_dbfs);
}

// new scaling value set -> display in label and set gain_scaler_rf (for waterfalls)
void MainWindow::get_frontend_scaling_value(int true_scaler_value_db) {
    // show true value next to slider
    if (true_scaler_value_db >= 0)
        ui -> rfscaling_label -> setText("+" + QString::number(true_scaler_value_db, 10) + " dB");
    else
        ui -> rfscaling_label -> setText(QString::number(true_scaler_value_db, 10) + " dB");

    gain_scaler_rf = true_scaler_value_db;
}


// =====================================================================================================================
//   DEBUG FUNCTIONALITY
// =====================================================================================================================

// =======================
// TESTGENERATOR

// single frequency selected -> read frequency from lineEdit and send to Testgenerator
void MainWindow::debug_single_selected() {
    QString req_freq_str = ui -> rftest_frequency_lineEdit -> text();
    double req_freq = req_freq_str.toDouble();
    emit this->send_debug_single(req_freq);
}

// sweep frequency selected -> read sweep properties and send to Testgenerator
void MainWindow::debug_sweep_selected() {
    QString start_freq_str = ui -> rftest_start_freq_lineEdit -> text();
    double test_sweep_start_freq = start_freq_str.toDouble();

    QString stop_freq_str = ui -> rftest_stop_freq_lineEdit -> text();
    double test_sweep_stop_freq = stop_freq_str.toDouble();

    QString num_freq_step_str = ui -> rftest_freq_step_lineEdit -> text();
    double test_sweep_step_freq = (test_sweep_stop_freq - test_sweep_start_freq) / num_freq_step_str.toDouble();

    emit this->send_debug_sweep(test_sweep_start_freq, test_sweep_stop_freq, test_sweep_step_freq);
}


// new frequency set -> show in GUI
void MainWindow::get_debug_true_frequency(double frequency) {
    ui -> true_rftest_freq_label -> setText(QString::number(frequency,'f',2));
}

// =======================
// OTHER DEBUG CORES

void MainWindow::on_debugmonitor_pushButton_clicked()
{
    // if window in opened the first time: stop waterfall and initialize debug monitor
    if (!debug_monitor_activated_) {
        ui_debug_monitor -> init();
        waterfall_ddc_ ->set_debug(true);
        waterfall_1a_ddc_ ->set_debug(true);
        waterfall_1b_ddc_ ->set_debug(true);
        debug_monitor_activated_ = true;
    }

    ui_debug_monitor -> show();

    statusBar()->showMessage("Radio control stopped, Debug Mode started");
}

void MainWindow::correct_plots() {
    audio_rx_->correct_size();
    waterfall_ddc_ -> correct_waterfall_size();
    waterfall_1a_ddc_ -> correct_waterfall_size();
    waterfall_1b_ddc_ -> correct_waterfall_size();
}

void MainWindow::waterfall_reset() {
    cerr << "assert reset IQ chain" << endl;
    iq_reset->write(0,0);

    cerr << "deassert reset IQ chain" << endl;
    iq_reset->write(0,0xFFFFFFFF);
}

void MainWindow::audio_reset() {
    cerr << "reset AUDIO" << endl;
    audio_rx_ -> reset();
}

// =====================================================================================================================
//   AUDIO RECEIVER FUNCTIONALITY
// =====================================================================================================================

// new audio level received -> display in thermo widget
void MainWindow::get_audio_out_level(int audio_level, int audio_level_dbfs) {
    ui -> audiomax_label -> setText(QString::number(audio_level, 10));
    ui -> audiomax_db_label -> setText("(" + QString::number(audio_level_dbfs, 10) + " dBFS)");

    if (audio_level_dbfs > -50)
        ui->audiomax_Thermo->setFillColor(QColor("red"));
    else
        ui->audiomax_Thermo->setFillColor(QColor("green"));

    ui->audiomax_Thermo->setValue(audio_level_dbfs);
}

// update the display of the bandwidth bars in the audio waterfalls
void MainWindow::set_bandwidth_bars() {

    if (ui->select_audio_ddc0_Button->isChecked()) {
        ui -> audio_narrow_bar_0_widget -> setVisible(ui->narrowband_radioButton->isChecked());
        ui -> audio_medium_bar_0_widget -> setVisible(ui->mediumband_radioButton->isChecked());
        ui -> audio_wide_bar_0_widget   -> setVisible(ui->wideband_radioButton->isChecked());
    }
    else {
        ui -> audio_narrow_bar_0_widget -> setVisible(false);
        ui -> audio_medium_bar_0_widget -> setVisible(false);
        ui -> audio_wide_bar_0_widget   -> setVisible(false);
    }

    if (ui->select_audio_ddc1_Button->isChecked()) {
        ui -> audio_narrow_bar_1_widget -> setVisible(ui->narrowband_radioButton->isChecked());
        ui -> audio_medium_bar_1_widget -> setVisible(ui->mediumband_radioButton->isChecked());
        ui -> audio_wide_bar_1_widget   -> setVisible(ui->wideband_radioButton->isChecked());
    }
    else {
        ui -> audio_narrow_bar_1_widget -> setVisible(false);
        ui -> audio_medium_bar_1_widget -> setVisible(false);
        ui -> audio_wide_bar_1_widget   -> setVisible(false);
    }
}

// sends current audio output properties from GUI (bw, spectrum inv., binaural) to audio rx
void MainWindow::get_audio_properties() {
    int bandwidth = 0;

    if (ui->wideband_radioButton->isChecked()) bandwidth = 0;
    if (ui->mediumband_radioButton->isChecked()) bandwidth = 1;
    if (ui->narrowband_radioButton->isChecked()) bandwidth = 2;

    set_bandwidth_bars();

    emit this->send_audio_output_properties(bandwidth, ui->invert_spectrum_checkBox->isChecked(), ui->binaural_checkBox->isChecked());
}

void MainWindow::on_select_audio_ddc0_Button_clicked() {
    set_bandwidth_bars();
    emit this->send_audio_ddc_selection(0, ui->select_audio_ddc0_Button->isChecked());
}

void MainWindow::on_select_audio_ddc1_Button_clicked() {
    set_bandwidth_bars();
    emit this->send_audio_ddc_selection(1, ui->select_audio_ddc1_Button->isChecked());
}

void MainWindow::on_audio_ddc0_freq_lineEdit_textChanged() {
    QString req_freq_str = ui -> audio_ddc0_freq_lineEdit -> text();    // read frequency from line edit
    double req_freq = req_freq_str.toDouble();
    emit this->send_audio_rx_frequency(req_freq, 0);    // set new frequency dor DDC 0
}

void MainWindow::on_audio_ddc1_freq_lineEdit_textChanged() {
    QString req_freq_str = ui -> audio_ddc1_freq_lineEdit -> text();    // read frequency from line edit
    double req_freq = req_freq_str.toDouble();
    emit this->send_audio_rx_frequency(req_freq, 1);    // set new frequency for DDC 1
}

void MainWindow::get_audio_frequency_clicked(double frequency) {
    // for the currently selected audio ddc:
    // set lineEdit for audio frequency and call the textchanged slots to change the frequency

    if (audio_rx_->get_ddc_output_active(1) && !audio_rx_->get_ddc_output_active(0)) {
        ui -> audio_ddc1_freq_lineEdit -> setText(QString::number(frequency,'f',0));
        emit this->send_audio_rx_frequency(frequency, 1);
    }
    else {
        ui -> audio_ddc0_freq_lineEdit -> setText(QString::number(frequency,'f',0));
        emit this->send_audio_rx_frequency(frequency, 0);
    }
}

// new frequency set for audio DDC 0 -> update line Edit for freuqency display
void MainWindow::get_audio_ddc0_new_frequency(double frequency){
    ui->audio_ddc0_freq_lineEdit->setText(QString::number(frequency,'f',0));
}

// new frequency set for audio DDC 0 -> update line Edit for freuqency display
void MainWindow::get_audio_ddc1_new_frequency(double frequency){
    ui->audio_ddc1_freq_lineEdit->setText(QString::number(frequency,'f',0));
}


// scaler moved -> send value in dB to audio scaler
void MainWindow::on_audioscaling_horizontalSlider_sliderMoved(int position){
    emit this -> send_audio_output_scaler_setting(position * 6);
}

// automatic scaling check box clicked -> send value to audio rx and enable or disable slider
void MainWindow::on_checkBox_audio_scale_auto_clicked(bool checked) {
    if (checked)
        ui->audioscaling_horizontalSlider->setDisabled(true);
    else
        ui->audioscaling_horizontalSlider->setDisabled(false);

    emit this -> send_audio_output_scaler_auto_setting(checked);
}

// new scaling value set -> display in label
void MainWindow::get_audio_scaling_value(int true_scaler_value_db) {
    // show true value next to slider
    if (true_scaler_value_db >= 0)
        ui -> audioscaling_label -> setText("+" + QString::number(true_scaler_value_db, 10) + " dB");
    else
        ui -> audioscaling_label -> setText(QString::number(true_scaler_value_db, 10) + " dB");
}



// =====================================================================================================================
//   Waterfall Settings
// =====================================================================================================================

void MainWindow::on_update_rate_comboBox_currentIndexChanged(int index) {
    switch(index) {
        case 0: timer_->start(5000); break;
        case 1: timer_->start(1000); break;
        case 2: timer_->start(500); break;
        case 3: timer_->start(250); break;
        case 4: timer_->start(125); break;
    }
}

void MainWindow::on_avg_ffts_comboBox_currentIndexChanged(const QString &arg1) {
    waterfall_ddc_->set_num_ffts(arg1.toInt());
    waterfall_1a_ddc_->set_num_ffts(arg1.toInt());
    waterfall_1b_ddc_->set_num_ffts(arg1.toInt());
}

void MainWindow::update_waterfall_color_map() {
    /*
     * for the color maps choose (always 50db dynamic range), values for range of -140 to 10
     */

    bool grey_mode = ui->colormap_mode_checkBox->isChecked();

    QwtLinearColorMap waterfall_colors_main(Qt::black, Qt::black);
    QwtLinearColorMap waterfall_colors_left(Qt::black, Qt::black);
    QwtLinearColorMap waterfall_colors_right(Qt::black, Qt::black);

    double color_scale_offset_main  = ui->colormap_main_ScrollBar->sliderPosition() * 0.05;
    double color_scale_offset_left  = ui->colormap_left_ScrollBar->sliderPosition() * 0.05;
    double color_scale_offset_right = ui->colormap_right_ScrollBar->sliderPosition() * 0.05;

    if (grey_mode) {
        waterfall_colors_main.setColorInterval(Qt::black, Qt::white);
        waterfall_colors_main.addColorStop(color_scale_offset_main, Qt::black);
        waterfall_colors_main.addColorStop(color_scale_offset_main + 0.333, Qt::white);

        waterfall_colors_left.setColorInterval(Qt::black, Qt::white);
        waterfall_colors_left.addColorStop(color_scale_offset_left, Qt::black);
        waterfall_colors_left.addColorStop(color_scale_offset_left + 0.333, Qt::white);

        waterfall_colors_right.setColorInterval(Qt::black, Qt::white);
        waterfall_colors_right.addColorStop(color_scale_offset_right, Qt::black);
        waterfall_colors_right.addColorStop(color_scale_offset_right + 0.333, Qt::white);
    }
    else {
        waterfall_colors_main.setColorInterval(Qt::blue, Qt::magenta);
        waterfall_colors_main.addColorStop(color_scale_offset_main+0.05, Qt::blue);
        waterfall_colors_main.addColorStop(color_scale_offset_main+0.1, Qt::green);
        waterfall_colors_main.addColorStop(color_scale_offset_main+0.2, Qt::yellow);
        waterfall_colors_main.addColorStop(color_scale_offset_main+0.3, Qt::red);
        waterfall_colors_main.addColorStop(color_scale_offset_main+0.333, Qt::magenta);

        waterfall_colors_left.setColorInterval(Qt::blue, Qt::magenta);
        waterfall_colors_left.addColorStop(color_scale_offset_left+0.05, Qt::blue);
        waterfall_colors_left.addColorStop(color_scale_offset_left+0.1, Qt::green);
        waterfall_colors_left.addColorStop(color_scale_offset_left+0.2, Qt::yellow);
        waterfall_colors_left.addColorStop(color_scale_offset_left+0.3, Qt::red);
        waterfall_colors_left.addColorStop(color_scale_offset_left+0.333, Qt::magenta);

        waterfall_colors_right.setColorInterval(Qt::blue, Qt::magenta);
        waterfall_colors_right.addColorStop(color_scale_offset_right+0.05, Qt::blue);
        waterfall_colors_right.addColorStop(color_scale_offset_right+0.1, Qt::green);
        waterfall_colors_right.addColorStop(color_scale_offset_right+0.2, Qt::yellow);
        waterfall_colors_right.addColorStop(color_scale_offset_right+0.3, Qt::red);
        waterfall_colors_right.addColorStop(color_scale_offset_right+0.333, Qt::magenta);
    }

    waterfall_ddc_ -> set_color_map(waterfall_colors_main);
    waterfall_1a_ddc_ -> set_color_map(waterfall_colors_left);
    waterfall_1b_ddc_ -> set_color_map(waterfall_colors_right);
}

void MainWindow::on_record_pushButton_clicked()
{
    if (ui->record_pushButton->isChecked()) {
        // start recording
        QString q_file_name = QFileDialog::getSaveFileName(this, tr("Open File for Recording"), "~/");
        string file_name = q_file_name.toLocal8Bit().constData();
        record_file_.open(file_name.c_str(),ios::out);
        currently_recording_ = true;
    }
    else {
        // stop recording
        currently_recording_ = false;
        record_file_.close();
    }

}
