#include "testgenerator.h"

Testgenerator::Testgenerator(QObject *parent) : QObject(parent) {
    test_generator_ = NULL;
    input_mux_ctrl_ = NULL;


    test_sweep_start_freq_ = 0;
    test_sweep_stop_freq_ = 0;
    test_sweep_step_freq_ = 0;
    test_sweep_frequency_= 0;
    sweep_active_ = false;
    sweep_stopped_ = false;

    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(sweep_timer_slot()));
    timer->start(1000);
}

Testgenerator::~Testgenerator() {
    if (test_generator_) delete test_generator_;
    if (input_mux_ctrl_) delete input_mux_ctrl_;
}

void Testgenerator::init_input_mux(int address, int memsize,int init_value) {
    if (input_mux_ctrl_) delete input_mux_ctrl_;
    input_mux_ctrl_ = new IP_Driver(address, memsize);
    input_mux_ctrl_->write(0x0,init_value);
}

void Testgenerator::init_generator(int address, int memsize, double master_frequency, int phase_width) {
    if (test_generator_) delete test_generator_;
    test_generator_ = new DDS(address, memsize, master_frequency, phase_width);
}

void Testgenerator::get_signal_source(int source) {
    switch (source) {
        case 0: input_mux_ctrl_ -> write(0x0, 0); break;
        case 1: input_mux_ctrl_ -> write(0x0, 1); break;
        case 2: input_mux_ctrl_ -> write(0x0, 3); break;
    }
}

void Testgenerator::get_single(double frequency) {
    sweep_active_ = false;
    double true_freq = test_generator_ -> set_frequency(frequency);
    emit this -> send_true_frequency(true_freq);
}


void Testgenerator::get_sweep(double start_f, double stop_f, double step_f) {
    sweep_active_ = true;
    test_sweep_frequency_ = start_f;
    test_sweep_start_freq_ = start_f;
    test_sweep_stop_freq_ = stop_f;
    test_sweep_step_freq_ = step_f;
}

void Testgenerator::get_sweep_stopped(bool stopped) {
    sweep_stopped_ = stopped;
}

void Testgenerator::sweep_timer_slot() {
    if (sweep_active_ && !sweep_stopped_) {
        test_sweep_frequency_= test_sweep_frequency_+ test_sweep_step_freq_;
        if (test_sweep_frequency_> test_sweep_stop_freq_) test_sweep_frequency_= test_sweep_start_freq_;
        emit this->send_true_frequency(test_sweep_frequency_);
    }
}
