#include "waterfalldata.h"

WaterfallData::WaterfallData(int data_array_xsize, int data_array_ysize)
{
    if (data_array_xsize % 2 == 1) cerr << "WARNING: waterfalldata.cpp: uneven data_array_xsize: zoom functionality is only tested for even data_array_xsizes" << endl;

    data_array_xsize_ = data_array_xsize;      // size of the data array
    data_array_ysize_ = data_array_ysize;
    background_value_ = -200;

    data_array_ = new double*[data_array_ysize_];

    for (int i=0; i<data_array_ysize_; i++)
        data_array_[i] = new double[data_array_xsize_];

    delete_all_data();

    // this code generates a test picture for debugging
    /*
    for (int j=0; j < data_array_xsize; j++)
        data_array_[100][j] = 0;

    for (int i=0; i < data_array_ysize; i++) {
        data_array_[i][82]  = -70;
        data_array_[i][164] = -70;
        data_array_[i][100] = -40;
        data_array_[i][150] = -50;
        data_array_[i][700] = -60;
        data_array_[i][900] = -80;
    } */

    scroll_pointer_temp_= NULL;
}

WaterfallData::~WaterfallData() {
    for (int i=0; i<data_array_ysize_; i++)
        delete[] data_array_[i];

    delete[] data_array_;
}

void WaterfallData::set_next_data_line(double *array) {
    scroll_pointer_temp_ = data_array_[0];

    for (int i=0; i < data_array_ysize_-1; i++)
        data_array_[i] = data_array_[i+1];

    data_array_[data_array_ysize_-1] = scroll_pointer_temp_;

    for (int i=0; i < data_array_xsize_; i++)
        data_array_[data_array_ysize_-1][i] = array[i];
}



void WaterfallData::shift_data_array (int shift_value) {

    if (shift_value > 0) {
        // shift left (mouse drag to right), copy right part of the array and append a dark part on the right hand side
        for (int i=0; i < data_array_ysize_; i++) {

            for (int j=0; j < (data_array_xsize_ - shift_value); j++)
                data_array_[i][j] = data_array_[i][j+shift_value];

            for (int j= (data_array_xsize_ - shift_value); j < data_array_xsize_; j++)
                data_array_[i][j] = background_value_;

        }

    }
    else {
        // shift right (mouse drag to left), copy left part of the array and append a dark part on the left side
        shift_value = -shift_value;

        for (int i=0; i < data_array_ysize_; i++) {

            for (int j=(data_array_xsize_-1); j >= shift_value; j--)
                data_array_[i][j] = data_array_[i][j-shift_value];

            for (int j= 0; j < shift_value; j++)
                data_array_[i][j] = background_value_;
        }
    }
}



void WaterfallData:: zoom_data_in(int center_idx) {                 // magnifies view on data, i.e. spread out data on x axis by a factor of 2 -> "blurry" image
    if ((center_idx < 0) || (center_idx > data_array_xsize_-1)) { cerr << "waterfalldata.cpp: zoom_data_in: argument center_idx invalid" << endl; exit(0); }

    for (int i=0; i < data_array_ysize_; i++) {

        for (int j = data_array_xsize_-1; j >= center_idx; j--)      // change part right of zoom center position
            data_array_[i][j] = data_array_[i][(center_idx+j)/2];

        for (int j = 0; j < center_idx; j++)                         // change part left of zoom center position
            data_array_[i][j] = data_array_[i][(center_idx+j)/2];

    }
}



void WaterfallData:: zoom_data_out(int center_idx) {

	//comments contain status messages for debugging the zoom functionality
	
    //cerr << "------------ zoom_data_out -----------------" << endl;
    //cerr << "center idx = " << center_idx << " = " <<125*(center_idx)/1024 << " MHz"<< endl;

    for (int i=0; i < data_array_ysize_; i++) {

        for (int j = 1 ; j < (data_array_xsize_-center_idx)/2; j++) {            // change part right of zoom center position
            if ((center_idx + 2*j + 1) > (data_array_xsize_-1)) cerr << "zoom_data_out exceeds " << data_array_xsize_-1 << endl;
            data_array_[i][center_idx+j] = max( data_array_[i][center_idx + 2*j - 1], data_array_[i][center_idx + 2*j] );
            //if (i==0) cerr << "data_array[" << center_idx+j << "] = max(" << center_idx + 2*j - 1 << ", " << center_idx + 2*j << ")" << endl;
        }


        for (int j = (data_array_xsize_+center_idx)/2; j < data_array_xsize_; j++)  {            // rightmost part: black
            data_array_[i][j] = background_value_;
            //if (i==0) cerr << "data_array[" << j << "] = schwarz" << endl;
        }

        for (int j = 0; j < (center_idx/2) ; j++) {                       // change part left of zoom center position
            if ((center_idx - 2*j - 1) < 0) cerr << "zoom_data_out exceeds 0, line " << i << endl;
            data_array_[i][center_idx - j] = max( data_array_[i][center_idx - 2*j], data_array_[i][center_idx - 2*j - 1] );
            //if (i==0) cerr << "data_array[" << center_idx - j << "] = max(" << center_idx - 2*j << ", " << center_idx - 2*j - 1 << ")" << endl;
        }


        for (int j=0; j< (center_idx/2 + 2); j++) {           // leftmost part: black
            data_array_[i][j] = background_value_;
            //if (i==0) cerr << "data_array[" << j << "] = schwarz" << endl;
        }
    }

    //cerr << " 0 bis " << (center_idx/2 + 2) << " = " <<  125*(center_idx/2 + 2)/1024 << "MHz, will be black" << endl;
}


void WaterfallData::delete_all_data() {
    for (int i=0; i < data_array_ysize_; i++)
        for (int j=0; j < data_array_xsize_; j++)
            data_array_[i][j] = background_value_;
}
