#ifndef WATERFALLDATA_H
#define WATERFALLDATA_H

#include <iostream>
#include <QTime>

using namespace std;

/** @brief holds a 2 dimensional array for display in a Waterfall Plot (instanciated by class WaterfallPlot) */
class WaterfallData
{
    double **data_array_;               // 2 dimensional data array
    double *scroll_pointer_temp_;       // temporary pointer to a row, required to implement waterfall "scrolling"

    int data_array_xsize_;              // size of the data array x
    int data_array_ysize_;              // y

    double background_value_;           // value for "background", i.e. areas with no information

public:
    /** @brief constructor: allocates memory, sets default background value to -200
     *  @param data_array_xsize width of data array
     *  @param data_array_ysize height of data array  */
    WaterfallData(int data_array_xsize, int data_array_ysize);

    /** @brief destruktor: frees memory of 2 dim array */
    ~WaterfallData();

    /** @brief set the value for "background" areas with no information, which occurs e.g. at zoom out (a default  value is set in code)
     *  @param background_value  */
    void set_background_value(double background_value) {background_value_ = background_value;}

    /** @brief sets a new line (first line) in the array and scrolls data in the array
     *  @param *array array of data of length data_array_xsize (a new line)  */
    void set_next_data_line(double *array);

    /** @brief accesses a pointer to the 2 dimensional data array
		returns pointer  */
    double** get_ptr_to_array() {return data_array_;}

    /** @brief shifts data, left-right (when waterfall is shifted)
     *  @param shift_value how far the plot is to be shifted (counted in array indices) */
    void shift_data_array (int shift_value);

    /** @brief zooms one step into data array (when waterfall is zoomed in)
     *  @param center_idx index of the array, where the zoom was centered on */
    void zoom_data_in(int center_idx);

    /** @brief zooms one step out of data array (when waterfall is zoomed out)
     *  @param center_idx index of the array, where the zoom was centered on */
    void zoom_data_out(int center_idx);

    /** @brief sets all values in data array to background color (empty waterfall) */
    void delete_all_data();
};

#endif // WATERFALLDATA_H
