
/*
  -------------------------------------------------------
    AD9467.cpp
    Main C++ class for AD9467 ADC
    Stefan Scholl, DC9ST, TU Kaiserslautern, 2014
  -------------------------------------------------------
*/


#include <iostream>
#include "AD9467.h"

using namespace std;


// private functions

// chooses the correct delay values for every single line
void AD9467::calibrate_delay_lines() {

	cerr << endl;
	cerr << "----------" << endl;
	cerr << "Calibration of the ADC Delay Interface" << endl << endl;

	int err_code_temp = 0;

	// bring ADC into test mode 7: zero/one toggling (all one and all zeros word alternatively)
	cerr << "switch ADC to test mode 7 (all zero/one word toggling)... ";

	err_code_temp = spi_iface_ -> set_test_mode(7); 

	if ((err_code_temp == 0) || (spi_iface_ -> get_test_mode() !=7))
		cerr << "ok" << endl;
	else
		cerr << "falied" << endl;

	cerr << endl;

	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// try all the 32 delay values and select the best ones
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	int test_ram_data[2048];			// captured data from test ram
	bool delay_value_ok[8][32];			// contains information, which delay values work on which data line, [delay line][delay value]


	for (int delay_value_counter = 0; delay_value_counter < 32; delay_value_counter++) {

		// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		// set current delay value for every of the 8 data lines
		// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


		err_code_temp = 0;

		for (int line_addr_counter = 0; line_addr_counter < 8; line_addr_counter++)
			err_code_temp = data_iface_ -> set_delay(line_addr_counter, delay_value_counter);
				
		

		if (err_code_temp != 0) cerr << "Error when setting up delay value" << delay_value_counter << " SPI error code: " << err_code_temp << endl;


		// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		//    capture test data in RAM and read into the PS
		// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


		data_iface_ -> capture_test_data();		// capture data from ADC into test RAM (PL)

		for (int test_data_counter = 0; test_data_counter < 2048; test_data_counter++)
			test_ram_data[test_data_counter] = data_iface_ -> read_test_data(test_data_counter);



		// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		//   check, if the captured data is correct
		// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

		int data_ok = 0xFFFF;
		int data_changed;

		// checks all ones/all zero pattern
		if ((test_ram_data[0] != 0xFFFF) & (test_ram_data[0] != 0x0000))
			data_ok = 0x0000;


		// checks if data changes every word
		for (int test_data_counter = 1; test_data_counter < 2048; test_data_counter++) {
			data_changed = test_ram_data[test_data_counter] ^ test_ram_data[test_data_counter-1];  // xor = 1 if data has changed, if xor = 0 data did not change -> error
			data_ok = data_ok & data_changed;
		}

		// look at data_ok, positions, where data_ok = 1 -> ok , positions where data_ok = 0 -> error
		for (int line_addr_counter = 0; line_addr_counter < 8; line_addr_counter++) {

			if ((data_ok>>line_addr_counter & 1) && (data_ok>>(line_addr_counter+8) & 1))
				delay_value_ok[line_addr_counter][delay_value_counter] = 1;
			else
				delay_value_ok[line_addr_counter][delay_value_counter] = 0;

		}

	};


	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	//   calculate and set final delay
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	cerr << "Overview of possible delay settings (number = working delay value, 'x' = non-working delay value)" << endl;

	int first_ok;
	int last_ok;
	int final_delay;

	// set final for every data line
	for (int line_addr_counter = 0; line_addr_counter < 8; line_addr_counter++) {

		cerr << "Line " << line_addr_counter << ":  ";

		first_ok = -1;
		last_ok = -1;

		int del_value = 0;

		// output information on all delay values over UART
		for (del_value = 0; del_value < 32; del_value++) {
			if (delay_value_ok[line_addr_counter][del_value] == 1)
				cerr << del_value << " ";  // hier evtl noch Dezimalstellen anpassen
			else
				cerr << " x ";
		}


		// look for (the leftmost) span of working delay values
		for (del_value = 0; del_value < 32; del_value++) {
			if ( (delay_value_ok[line_addr_counter][del_value] == 1) && (first_ok == -1) )
				first_ok = del_value;

			if ( (delay_value_ok[line_addr_counter][del_value] == 0) && (first_ok != -1) && (last_ok == -1))
				last_ok = del_value-1;
		}


		// set as delay value the middle between first and last working delay value
		final_delay = (first_ok + last_ok) / 2;
		err_code_temp = data_iface_ -> set_delay(line_addr_counter, final_delay);

		if (err_code_temp == 0)
			cerr << " - delay set: " << final_delay << " " << endl;				
		else
			cerr << " - ERROR: unable to set delay " << final_delay << " (error code " << err_code_temp << ")" << endl;

	}

	// switch off ADC test mode:
	cerr << endl << "switch off ADC test mode... ";

	err_code_temp = spi_iface_ -> set_test_mode(0); 

	if (err_code_temp == 0)
		cerr << "ok, now in normal mode again" << endl;
	else
		cerr << "falied" << endl;



	cerr << endl;
	cerr << "calibration of the ADC delay interface done" << endl;
	cerr << "----------" << endl << endl;

};



// constructor and destructor

//constructor
AD9467::AD9467(int base_address_spi, int size_in_k_spi, int base_address_data, int size_in_k_data) {
	cerr << "initialize IP Cores for the ADC for SPI and Data Interface: " << endl;	
	spi_iface_ = new AD9467_SPI(base_address_spi, size_in_k_spi);
	data_iface_ = new AD9467_Data_Interface(base_address_data, size_in_k_data);
}


//destructor
AD9467::~AD9467() {
	delete spi_iface_;
	delete data_iface_;
}



// public functions

// init function, that calibrates the interface for the AD9467
int AD9467::init() {
	cerr << endl; 		
	cerr << "---------------------------------------------" << endl;
	cerr << "Initialization of the AD9467" << endl << endl;

	data_iface_ -> check_availability_on_fpga(); // check if hardware design is loaded into the FPGA	

	// init delay controller
	cerr << "init delay controller for ADC data interface... ";
	int error_code = data_iface_ -> init_delay_controller();

	if (error_code == 0)
		cerr << "successful" << endl;
	else
		cerr << "failed" << endl;


	// reset ADC
	cerr << "resetting ADC via SPI... ";
	spi_iface_ -> reset_adc();
	cerr << "done" << endl;

	volatile int a = 0;

	for (int j=0; j < 100; j++)
		for (int i=0; i<50000; i++)
			a += 1;


	// check if SPI works (reading CHIP ID)
	cerr << "checking SPI: reading chip ID: ";
	int chip_id = spi_iface_ -> get_chip_id();
	cerr << "0x" << hex << chip_id << dec;

	if (chip_id == 0x50)
		cerr << ", ok" << endl;
	else
		cerr << ", failed !" << endl;


	// calibrate delay lines
	calibrate_delay_lines();

	// set output format to 2s complement
	cerr << "set output format to 2s complement" <<endl;
	spi_iface_ -> set_data_format(1);

	cerr << endl;
	cerr << "initialization of the AD9467 done" << endl;
	cerr << "---------------------------------------------" << endl << endl;
	
	return 0;
}

		


	

