#ifndef AD9467_H
#define AD9467_H

/*
  -------------------------------------------------------
    AD9467.h
    Main C++ driver class for AD9467 ADC
    Stefan Scholl, DC9ST, TU Kaiserslautern, 2014
  -------------------------------------------------------
*/


#include "AD9467_SPI.h"
#include "AD9467_Data_Interface.h"

/** @brief Main Driver for the AD9467  */
class AD9467 {

private:
	
	/** @brief chooses the correct delay values for every single line */
	void calibrate_delay_lines();

 
	AD9467_SPI 				*spi_iface_;		// driver of spi interface
	AD9467_Data_Interface 	*data_iface_;		// driver for data interface

public:
	/** @brief class constructor
	 *  @param base_adress_spi address of the spi core (from Vivado)
	 *  @param size_in_k_spi size of the required adress space in k for the SPI core(from Vivado)
	 *  @param base_adress_data address of the data interface core (from Vivado)
	 *  @param size_in_k_data size of the required adress space in k for the data interface core(from Vivado) 	 */
	AD9467(int base_address_spi, int size_in_k_spi, int base_address_data, int size_in_k_data);


	/** @brief class deconstructor */	
	~AD9467();


	/** @brief init function, that calibrates the interface for the AD9467
	    @return error code: 0: no error 	*/
	int init();
	
};

#endif /* AD9467_H */
