#ifndef AD9467_DATA_INTERFACE_H
#define AD9467_DATA_INTERFACE_H

/*
  -------------------------------------------------------
    AD9467_Data_Interface.h
    C++ driver class for ADC data interface (IO modules)
    Stefan Scholl, DC9ST, TU Kaiserslautern, 2014
  -------------------------------------------------------
*/

#include "IP_Driver.h"


/** @brief Driver for the data interface of the AD9467  */
class AD9467_Data_Interface {

private:
	
	/** @brief wait for approx. t_ms ms */
	void wait_ms(int t_ms);
 
	IP_Driver *data_iface_core_; // low level driver

public:
	/** @brief class constructor
	 *  @param base_adress address of the spi core (from Vivado), e.g. 0x44C00000
	 *  @param size_in_k size of the required adress space in k (from Vivado)	 */
	AD9467_Data_Interface(int base_address, int size_in_k);

	/** @brief class deconstructor */	
	~AD9467_Data_Interface();


	/** @brief sets delay value for one delay element
	    @param address of I/O line (0 to 7)
	    @param delay value to be written to delay module (0 to 31);
	    @return error code: 0: no error, -1: timeout, -2: invalid delay value, -3: invalid line address  	*/
	int set_delay(int line_addr, int delay_value);


	/** @brief captures data from the ADC to the test data memory
	 *  @return int 8 bit chip ID	 */
	void capture_test_data();


	/** @brief reads one sample of data from the test data memory
	    @param address of test data memory data word to be read
	 *  @return test sample (16 bit)	 */
	int read_test_data(int addr);

	
	/** @brief inits the delay controller (necessary for the delay controller)
	*   @return error code: 0: no error, 1: timeout errror	*/
	int init_delay_controller();

	/** @brief check core availability on FPGA (i.e. shows if hardware design is loaded) */
	void check_availability_on_fpga();
};

#endif /* AD9467_DATA_INTERFACE_H */
