#ifndef AD9467_SPI_H
#define AD9467_SPI_H

/*
  -------------------------------------------------------
    AD9467_SPI.h
    SPI interface driver class for the spi_ad_0 core for the AD9467
    Stefan Scholl, DC9ST, TU Kaiserslautern, 2014
  -------------------------------------------------------
*/

#include "IP_Driver.h"

/** @brief Driver for the SPI interface of the AD9467 using the spi_ad_0 core  */
class AD9467_SPI {

private:
	/** @brief sends a single write instruction to the ADC */
	void spi_write(int address, int data);
	
	/** @brief sends a simple read instruction to the ADC */
	int spi_read(int address);
	 
	IP_Driver *spi_core_;	// low level driver

public:
	/** @brief class constructor
	 *  @param base_adress address of the spi core (from Vivado), e.g. 0x44C00000
	 *  @param size_in_k size of the required adress space in k (from Vivado)	 */
	AD9467_SPI(int spi_base_address, int size_in_k);

	/** @brief class deconstructor */	
	~AD9467_SPI();

	/** @brief resets ADC chip (soft reset)  */
	void reset_adc();

	/** @brief reads the chip ID (0x50 for the AD9467)
	 *  @return int 8 bit chip ID   */
	int get_chip_id();

	/** @brief reads the chip grade (0x20 for the AD9467-250)
	 *  @return int 8 bit value */
	int get_chip_grade();

	/** @brief reads the current test mode
	 *  @return test mode 	 */
	int get_test_mode();

 	/** @brief sets the test mode
	 *  @param test_mode (e.g. off:0, checkerboard:4, PN23:5, 1/0 word toggle: 7)
	 *  @return error code: 0: no error, -1: test_mode invalid   */
	int set_test_mode(int test_mode);

 	/** @brief sets the data format
	 *  @param format (offset binary(default):0, tows complement:1, Gray code:2)
	 *  @return error code: 0: no error, -1: format invalid   */
	int set_data_format(int format);
};

#endif /* AD9467_SPI_H */
