/*
  -------------------------------------------------------
    CIC.cpp
    CIC driver class
    Stefan Scholl, DC9ST, TU Kaiserslautern, 2014
  -------------------------------------------------------
*/

#include "IP_Driver.h"
#include "CIC.h"
#include <iostream>
#include <math.h> // for pow

using namespace std;

// constructor & deconstructor

// class constructor
CIC::CIC(int base_address, int size_in_k, int initial_rate) {
	cerr << "setup CIC IP core:" << endl;	
	cic_iface_core_ = new IP_Driver(base_address, size_in_k);
	set_rate(initial_rate);
}

// class deconstructor	
CIC::~CIC() {
	delete cic_iface_core_;
}


// public functions

//sets CIC rate
int CIC::set_rate(int rate) {

	// is specified rate a valid rate?
	if ((rate < 4) || (rate > pow(2,16)) ) {
		cerr << "invalid rate ("<< rate <<") specified" << endl;
		return -2;
	}

	int cic_instruction = 0x80000000 | rate;			// 0x80000000 add the valid bit

	cic_iface_core_ -> write(0x00, cic_instruction);  	// send instruction to CIC
	
	// wait for transaction to complete
	int timeout_counter = 1000000;

	do { timeout_counter--;}
	while ( (cic_iface_core_ -> read(0x04) != 1) && (timeout_counter > 0) );

	cic_iface_core_ -> write(0x00, 0x00000000);  		// clear instruction
		
	if (timeout_counter <= 0) return -1;  				// catch timeout	

	decimation_rate_ = rate;

	return 0;
}
