#ifndef CIC_H
#define CIC_H

/*
  ---------------------------------------------------------------------------------------------------
    CIC.h
    CIC interface driver class for the Xilinx CIC core
    with AXI -> AXIS core (dds_axi_interface) as interface to the Xilinx CIC core
    Stefan Scholl, DC9ST, TU Kaiserslautern, 2015
  ---------------------------------------------------------------------------------------------------
*/

#include "IP_Driver.h"

/** @brief Driver for the Xilinx CIC core using the axi_dds_interface core  */
class CIC {

private:

	IP_Driver *cic_iface_core_;
	int decimation_rate_;

public:
	/** @brief class constructor
	 *  @param base_adress address of the spi core (from Vivado), e.g. 0x44C00000
	 *  @param size_in_k size of the required adress space in k (from Vivado) */
	CIC(int base_address, int size_in_k, int initial_rate);

	/** @brief class deconstructor */	
	~CIC();

	/** @brief sets CIC decimation rate
	  * @return  error code (0:ok, -1:timeout/no response, -2: invalid rate) */
	int set_rate(int rate);

	/** @brief gets CIC decimation rate
	 *  @returns decimation rate */
	int get_rate() {return decimation_rate_;}
};

#endif /* CIC_H */
