#include "Data_Capture_Dual.h"
#include <stdlib.h>
#include <iostream>

Data_Capture_Dual::Data_Capture_Dual(int base_address, int size_in_k, int num_captured_words) : Data_Capture (base_address, size_in_k, num_captured_words) {
    data_array_int2double_2_ = new int[num_captured_words_];
}

Data_Capture_Dual::~Data_Capture_Dual() {
    delete[] data_array_int2double_2_;
}

int Data_Capture_Dual::read(int *data_array_1, int *data_array_2) {

    if (data_capture_core_ -> read(0x04) == 1) return -2; // capture module is busy and should not be read

    int check_address;

    for (int i=0; i < num_captured_words_; i++) {
        timeout_counter_ = 0;

        data_capture_core_ -> write(0x14, i);  // request address for read

        do {
            check_address = data_capture_core_ -> read(0x18);
            timeout_counter_++;
        }
        while ( (check_address != i) && (timeout_counter_ < 1000) ); // wait until address is applied

        if (timeout_counter_ == 1000) return -1;

        data_array_1[i] = data_capture_core_ -> read(0x10);
        data_array_2[i] = data_capture_core_ -> read(0x08);
    }

    return 0;
}


int Data_Capture_Dual::capture_and_read(int *data_array_1, int *data_array_2) {

    // capture data
    capture();

    // wait for capture to be finished
    int module_busy;
    timeout_counter_ = 0;

    do {
        module_busy = data_capture_core_ -> read(0x04);
        timeout_counter_ ++;
    }
    while ( (module_busy == 1) && (timeout_counter_ < timeout_max_) );

    if (timeout_counter_ == timeout_max_) return -3;

    // read the data
    int read_error = read(data_array_1, data_array_2);
    if (read_error == -1) return -1;

    return 0;
}


//overloaded version of above method for double arrays
int Data_Capture_Dual::capture_and_read(double *data_array_1, double *data_array_2) {
    int err_code = 0;
    err_code = capture_and_read(data_array_int2double_, data_array_int2double_2_);

    for (int i=0; i < num_captured_words_; i++) {
        data_array_1[i] = (double) data_array_int2double_[i];
        data_array_2[i] = (double) data_array_int2double_2_[i];
    }

    return err_code;
}


//overloaded version of above method for double arrays and smaller array size
int Data_Capture_Dual::capture_and_read(double *data_array_1, double *data_array_2, int size) {
    int err_code = 0;
    err_code = capture_and_read(data_array_int2double_, data_array_int2double_2_);

	if (num_captured_words_ < size) {
		std::cerr << "ERROR: Data_Capture_Dual.cpp: capture_and_read() specified array size (=" << size << ") larger than num_captured_words of IP Core" << std::endl;
		exit(1);
	}

    for (int i=0; i < size; i++) {
        data_array_1[i] = (double) data_array_int2double_[i];
        data_array_2[i] = (double) data_array_int2double_2_[i];
    }

    return err_code;
}
