#ifndef DATA_CAPTURE_DUAL_H
#define DATA_CAPTURE_DUAL_H

#include "Data_Capture.h"

/** @brief Driver for Data Capture Module for dual channel operation  */
class Data_Capture_Dual : public Data_Capture
{
private:
    int *data_array_int2double_2_;  	//required for integer to double conversion in capture_and_read method of additional channel 2

public:
    /** @brief class constructor
     *  @param base_adress address of the spi core (from Vivado), e.g. 0x44C00000
     *  @param size_in_k size of the required adress space in k (from Vivado)
     *  @param num_captured_words number of words to be captured (from IP core properties in Vivado)	 */
    Data_Capture_Dual(int base_address, int size_in_k, int num_captured_words);

    /** @brief class deconstructor */
    ~Data_Capture_Dual();

    /** @brief reads data, that has been captured previously by capture() (non blocking)
     *  @param data_array_1 pointer to a data array for channel 1, into which the function writes the captured data
     *  @param data_array_2 pointer to a data array for channel 2, into which the function writes the captured data
     *  @return 0: no errors, -1: timeout during read (maybe IP addess wrong), -2: module busy / capture in progess  */
    int read(int *data_array_1, int *data_array_2);

    /** @brief captures data and reads it out (blocking), for integer array
     *  @param data_array_1 pointer to a data array for channel 1, into which the function writes the captured data
     *  @param data_array_2 pointer to a data array for channel 2, into which the function writes the captured data
     *  @return 0: no errors, -1: timeout during read (maybe IP addess wrong), -3: capture timeout = capture takes too long (read not possible)  */
    int capture_and_read(int *data_array_1, int *data_array_2);

    /** @brief captures data and reads it out (blocking), overloaded for double array
     *  @param data_array_1 pointer to a data array for channel 1, into which the function writes the captured data
     *  @param data_array_2 pointer to a data array for channel 2, into which the function writes the captured data
     *  @return 0: no errors, -1: timeout during read (maybe IP addess wrong), -3: capture timeout = capture takes too long (read not possible)  */
    int capture_and_read(double *data_array_1, double *data_array_2);

    /** @brief captures data and reads it out (blocking), overloaded for double array and smaller array size
     *  @param data_array_1 pointer to a data array for channel 1, into which the function writes the captured data
     *  @param data_array_2 pointer to a data array for channel 2, into which the function writes the captured data
     *  @param size array sizes of data_array_1 and _2
     *  @return 0: no errors, -1: timeout during read (maybe IP addess wrong), -3: capture timeout = capture takes too long (read not possible)  */
    int capture_and_read(double *data_array_1, double *data_array_2, int size);
};

#endif // DATA_CAPTURE_DUAL_H
