/*
  -------------------------------------------------------
    Signal_Scaler.cpp
    Signal_Sclaer class (ATT & AMP)
    Stefan Scholl, DC9ST, TU Kaiserslautern, 2014
  -------------------------------------------------------
*/

#include "IP_Driver.h"
#include "Signal_Scaler.h"
#include <iostream>
#include <cmath>

using namespace std;

// constructor & deconstructor

// class constructor
Signal_Scaler::Signal_Scaler(int base_address, int size_in_k, int max_scaling_value_db) {
	cerr << "setup Signal Scaler IP core:" << endl;
	att_amp_core_ = new IP_Driver(base_address, size_in_k);
	max_scaling_db_ = max_scaling_value_db;
}

// class deconstructor	
Signal_Scaler::~Signal_Scaler() {
	delete att_amp_core_;
}



// public functions

//sets Signal_Scaler attenuation or amplification
int Signal_Scaler::set_scaling(int scaling_in_db) {

	if ( ((scaling_in_db % 6) != 0) || ( abs(scaling_in_db) > max_scaling_db_) ) {
		cerr << "Signal_Scaler.cpp: invalid value for scaling (" << scaling_in_db << "), only values in 6 db steps allowed, maximum value +/-:" << max_scaling_db_ << endl;
		return 1;	
	}

	if (scaling_in_db > max_scaling_db_) scaling_in_db = max_scaling_db_;
	if (scaling_in_db < -max_scaling_db_) scaling_in_db = -max_scaling_db_;

	int scaling_step = scaling_in_db / 6;

	if (scaling_in_db > 0) {
			
		// AMPLIFICATION

		// amplification
		att_amp_core_->write(0x04, scaling_step);

		if (att_amp_core_->read(0x04) != scaling_step) {
			cerr << "Signal_Scaler.cpp: could not write AMP value" << endl;
			return 2;
		}

		// attenuation=0
		att_amp_core_->write(0x00, 0);

		if (att_amp_core_->read(0x00) != 0) {
			cerr << "Signal_Scaler.cpp: could not write ATT value" << endl;
			return 2;
		}
	}
	else {
		// ATTENUATION

		// amplification = 0
		att_amp_core_->write(0x04, 0);

		if (att_amp_core_->read(0x04) != 0) {
			cerr << "Signal_Scaler.cpp: could not write AMP value" << endl;
			return 2;
		}

		// attenuation
		att_amp_core_->write(0x00, -scaling_step);

		if (att_amp_core_->read(0x00) != -scaling_step) {
			cerr << "Signal_Scaler.cpp: could not write ATT value" << endl;
			return 2;
		}
	}

	return 0;
}

int Signal_Scaler::get_max_value() {
	return att_amp_core_ -> read(0x1c);
}

int Signal_Scaler::reset_max_value() {
	
	// apply reset	
	att_amp_core_ -> write(0x08, 1);

	if (att_amp_core_->read(0x08) != 1) {
		cerr << "Signal_Scaler.cpp: could not reset max value" << endl;
		return 1;
	}

	// clear reset	
	att_amp_core_ -> write(0x08, 0);

	if (att_amp_core_->read(0x08) != 0) {
		cerr << "Signal_Scaler.cpp: could not reset max value" << endl;
		return 1;
	}
	
	return 0;
}





