#ifndef SIGNAL_SCALER_H
#define SIGNAL_SCALER_H

/*
  ---------------------------------------------------------------------------------------------------
    Signal_Scaler.h
    Signal_Scaler driver class for the Signal_Scaler IP core
    IP Core consists of an ATT followed by an AMP
    Stefan Scholl, DC9ST, TU Kaiserslautern, 2015
  ---------------------------------------------------------------------------------------------------
*/

#include "IP_Driver.h"

/** @brief Driver for the Signal_Scaler IP core */
class Signal_Scaler {

private:

	IP_Driver 	*att_amp_core_;
	int 		max_scaling_db_;

public:
	/** @brief class constructor
	 *  @param base_adress address of the spi core (from Vivado), e.g. 0x44C00000
	 *  @param size_in_k size of the required adress space in k (from Vivado)
	 *  @param max_scaling_value_db maximum scaling value in dB (e.g. 48 means -48 dB to +48 dB)  */
	Signal_Scaler(int base_address, int size_in_k, int max_scaling_value_db);

	/** @brief class deconstructor */	
	~Signal_Scaler();

	/** @brief sets scaling, <0dB: attenuation, >0dB: -> amplification (in 6 dB steps (shifting))
	  * @return error code (0=successful)  */
	int set_scaling(int scaling_in_db);

	/** @brief reads maximum occured absolute value at the module's input since last call of reset_max_value()
	  * @return maximum abs value */
	int get_max_value();

	/** @brief resets the maximum abs value
	  * @return error code (0=successful, 1=could not reset) */
	int reset_max_value();

};

#endif /* SIGNAL_SCALER_H */
